import argparse
import sys
import signal
from argparse import ArgumentError
from conans.client.command import (Command as Command_co, SmartFormatter, OnceArgument, 
                                   _PATH_HELP, _INSTALL_FOLDER_HELP, 
                                   _SOURCE_FOLDER_HELP, _BUILD_FOLDER_HELP, 
                                   _PATTERN_REF_OR_PREF_HELP, _QUERY_HELP)
from sense2 import __version__ as client_version
from sense2.client.sense2_api import Sense2, default_manifest_folder, _make_abs_path, ProfileData
from conans.errors import ConanException, ConanInvalidConfiguration, NoRemoteAvailable, \
    ConanMigrationError, ConanInvalidSystemRequirements
from conans.cli.exit_codes import SUCCESS, ERROR_MIGRATION, ERROR_GENERAL, USER_CTRL_C, \
    ERROR_SIGTERM, USER_CTRL_BREAK, ERROR_INVALID_CONFIGURATION, ERROR_INVALID_SYSTEM_REQUIREMENTS
from conans.model.ref import ConanFileReference, PackageReference, get_reference_fields, \
    check_valid_ref
from conans.client.output import Color
from conans.client.cmd.uploader import UPLOAD_POLICY_FORCE, \
    UPLOAD_POLICY_NO_OVERWRITE, UPLOAD_POLICY_NO_OVERWRITE_RECIPE, UPLOAD_POLICY_SKIP

class Command(Command_co):
    """A single command of the conan application, with all the first level commands. Manages the
    parsing of parameters and delegates functionality in collaborators. It can also show the
    help of the tool.
    """
    def __init__(self, sense2_api):
        assert isinstance(sense2_api, Sense2)
        self._conan = sense2_api
        self._out = sense2_api.out


    def install_deb(self, *args):
        """
        Calls your local conanfile.py 'install_deb()' method.
        """

        parser = argparse.ArgumentParser(description=self.build.__doc__,
                                         prog="sense2 install_deb",
                                         formatter_class=SmartFormatter)
        parser.add_argument("conanfile_path", help=_PATH_HELP)
        args = parser.parse_args(*args)
        self._warn_python_version()
        
        return self._conan.install_deb(conanfile_path=args.conanfile_path)

    def install_docker(self, *args):
        """
        Calls your local conanfile.py 'install_docker()' method.
        """

        parser = argparse.ArgumentParser(description=self.build.__doc__,
                                         prog="sense2 install_docker",
                                         formatter_class=SmartFormatter)
        parser.add_argument("conanfile_path", help=_PATH_HELP)

        args = parser.parse_args(*args)

        self._warn_python_version()
        
        return self._conan.install_docker(conanfile_path=args.conanfile_path)

    def install_sense_system(self, *args):
        """
        Calls your local conanfile.py 'install_sense_system()' method.
        """

        parser = argparse.ArgumentParser(description=self.build.__doc__,
                                         prog="sense2 install_sense_system",
                                         formatter_class=SmartFormatter)
        parser.add_argument("conanfile_path", help=_PATH_HELP)

        args = parser.parse_args(*args)

        self._warn_python_version()
        
        return self._conan.install_sense_system(conanfile_path=args.conanfile_path)
    
    def deb(self, *args):
        """
        Calls your local conanfile.py 'deb()' method.

        This command works in the user space and it will copy artifacts from
        the --build-folder and --source-folder folder to the --package-folder
        one.  It won't create a new package in the local cache, if you want to
        do it, use 'conan create' or 'conan export-pkg' after a 'conan build'
        command.
        """
        parser = argparse.ArgumentParser(description=self.package.__doc__,
                                         prog="sense2 deb",
                                         formatter_class=SmartFormatter)
        parser.add_argument("path", help=_PATH_HELP)
        parser.add_argument("-bf", "--build-folder", action=OnceArgument, help=_BUILD_FOLDER_HELP)
        parser.add_argument("-pf", "--deb-folder", action=OnceArgument,
                            help="folder to install the package. Defaulted to the "
                                 "'{build_folder}/package' folder. A relative path can be specified"
                                 " (relative to the current directory). Also an absolute path"
                                 " is allowed.")
        parser.add_argument("-sf", "--source-folder", action=OnceArgument, help=_SOURCE_FOLDER_HELP)
        parser.add_argument("-if", "--install-folder", action=OnceArgument,
                            help=_INSTALL_FOLDER_HELP)
        
        args = parser.parse_args(*args)
        try:
            if "@" in args.path and ConanFileReference.loads(args.path):
                raise ArgumentError(None,
                                    "'sense2 deb' doesn't accept a reference anymore. "
                                    "The path parameter should be a conanfile.py or a folder "
                                    "containing one. If you were using the 'conan package' "
                                    "command for development purposes we recommend to use "
                                    "the local development commands: 'conan build' + "
                                    "'conan package' and finally 'conan create' to regenerate the "
                                    "package, or 'conan export_package' to store the already built "
                                    "binaries in the local cache without rebuilding them.")
        except ConanException:
            pass

        self._warn_python_version()
        return self._conan.deb(path=args.path,
                                build_folder=args.build_folder,
                                deb_folder=args.deb_folder,
                                source_folder=args.source_folder,
                                install_folder=args.install_folder)
    
    def upload_deb(self, *args):
        """
        Uploads a recipe and binary packages to a remote.

        If no remote is specified, the first configured remote (by default conan-center, use
        'conan remote list' to list the remotes) will be used.
        """
        parser = argparse.ArgumentParser(description=self.upload.__doc__,
                                         prog="sense2 upload_deb",
                                         formatter_class=SmartFormatter)
        parser.add_argument('pattern_or_reference', help=_PATTERN_REF_OR_PREF_HELP)
        parser.add_argument("-p", "--package", default=None,
                            help="Package ID [DEPRECATED: use full reference instead]",
                            action=OnceArgument)
        parser.add_argument('-q', '--query', default=None, action=OnceArgument,
                            help="Only upload packages matching a specific query. " + _QUERY_HELP)
        parser.add_argument("-r", "--remote", action=OnceArgument,
                            help='upload to this specific remote')
        parser.add_argument("--all", action='store_true', default=True,
                            help='Upload both package recipe and packages')
        parser.add_argument("--skip-upload", action='store_true', default=False,
                            help='Do not upload anything, just run the checks and the compression')
        parser.add_argument("--force", action='store_true', default=False,
                            help='Ignore checks before uploading the recipe: it will bypass missing'
                                 ' fields in the scm attribute and it will override remote recipe'
                                 ' with local regardless of recipe date')
        parser.add_argument("--check", action='store_true', default=False,
                            help='Perform an integrity check, using the manifests, before upload')
        parser.add_argument('-c', '--confirm', default=False, action='store_true',
                            help='Upload all matching recipes without confirmation')
        parser.add_argument('--retry', default=None, type=int, action=OnceArgument,
                            help="In case of fail retries to upload again the specified times.")
        parser.add_argument('--retry-wait', default=None, type=int, action=OnceArgument,
                            help='Waits specified seconds before retry again')
        parser.add_argument("-no", "--no-overwrite", nargs="?", type=str, choices=["all", "recipe"],
                            action=OnceArgument, const="all",
                            help="Uploads package only if recipe is the same as the remote one")
        parser.add_argument("-j", "--json", default=None, action=OnceArgument,
                            help='json file path where the upload information will be written to')
        parser.add_argument("--parallel", action='store_true', default=False,
                            help='Upload files in parallel using multiple threads. '
                                 'The default number of launched threads is set to the value of '
                                 'cpu_count and can be configured using the CONAN_CPU_COUNT '
                                 'environment variable or defining cpu_count in conan.conf')

        args = parser.parse_args(*args)

        try:
            pref = PackageReference.loads(args.pattern_or_reference, validate=True)
        except ConanException:
            reference = args.pattern_or_reference
            package_id = args.package

            if package_id:
                self._out.warn("Usage of `--package` argument is deprecated."
                               " Use a full reference instead: "
                               "`sense2 upload_deb [...] {}:{}`".format(reference, package_id))

            if args.query and package_id:
                raise ConanException("'--query' argument cannot be used together with '--package'")
        else:
            reference = repr(pref.ref)
            package_id = "{}#{}".format(pref.id, pref.revision) if pref.revision else pref.id

            if args.package:
                raise ConanException("Use a full package reference (preferred) or the `--package`"
                                     " command argument, but not both.")
            if args.query:
                raise ConanException("'--query' argument cannot be used together with "
                                     "full reference")

        if args.force and args.no_overwrite:
            raise ConanException("'--no-overwrite' argument cannot be used together with '--force'")
        if args.force and args.skip_upload:
            raise ConanException("'--skip-upload' argument cannot be used together with '--force'")
        if args.no_overwrite and args.skip_upload:
            raise ConanException("'--skip-upload' argument cannot be used together "
                                 "with '--no-overwrite'")

        self._warn_python_version()

        if args.force:
            policy = UPLOAD_POLICY_FORCE
        elif args.no_overwrite == "all":
            policy = UPLOAD_POLICY_NO_OVERWRITE
        elif args.no_overwrite == "recipe":
            policy = UPLOAD_POLICY_NO_OVERWRITE_RECIPE
        elif args.skip_upload:
            policy = UPLOAD_POLICY_SKIP
        else:
            policy = None

        info = None
        try:
            info = self._conan.upload_deb(pattern=reference, package=package_id,
                                      query=args.query, remote_name=args.remote,
                                      all_packages=args.all, policy=policy,
                                      confirm=args.confirm, retry=args.retry,
                                      retry_wait=args.retry_wait, integrity_check=args.check,
                                      parallel_upload=args.parallel)

        except ConanException as exc:
            info = exc.info
            raise
        finally:
            if args.json and info:
                self._outputer.json_output(info, args.json, os.getcwd())

      
    # def deploy_deb(self, *args):
    #     """
    #     Calls your local conanfile.py 'install_sense_system()' method.
    #     """
    #     parser = argparse.ArgumentParser(description=self.build.__doc__,
    #                                      prog="sense2 install_sense_system",
    #                                      formatter_class=SmartFormatter)
    #     parser.add_argument("conanfile_path", help=_PATH_HELP)
    #     args = parser.parse_args(*args)
    #     self._warn_python_version()
        
    #     return self._conan.deploy_deb(conanfile_path=args.conanfile_path)

    # def set_source(self, *args):
    #     """
    #     Calls your local conanfile.py 'set_source()' method.
    #     """

    #     parser = argparse.ArgumentParser(description=self.build.__doc__,
    #                                      prog="sense2 set_source",
    #                                      formatter_class=SmartFormatter)
    #     parser.add_argument("path", help=_PATH_HELP)
    #     parser.add_argument("-sf", "--source-folder", action=OnceArgument, help=_SOURCE_FOLDER_HELP)

    #     args = parser.parse_args(*args)

    #     self._warn_python_version()
        
    #     return self._conan.set_source(conanfile_path=args.path, source_folder=args.source_folder)
 
    def _show_help(self):
        """
        Prints a summary of all commands.
        """
        grps = [("Consumer commands", ("install", "config", "get", "info", "search")),
                ("Creator commands", ("new", "create", "upload", "export", "export-pkg", "test")),
                ("Package development commands", ("source", "build", "package", "editable",
                                                  "workspace")),
                ("deb development commands", ("install_sense_system", "deb", 
                                              "install_deb", "install_docker",
                                              "upload_deb")),
                ("Misc commands", ("profile", "remote", "user", "imports", "copy", "remove",
                                   "alias", "download", "inspect", "help", "lock", "frogarian"))]

        def check_all_commands_listed():
            """Keep updated the main directory, raise if don't"""
            all_commands = self._commands()
            all_in_grps = [command for _, command_list in grps for command in command_list]
            if set(all_in_grps) != set(all_commands):
                diff = set(all_commands) - set(all_in_grps)
                raise Exception("Some command is missing in the main help: %s" % ",".join(diff))
            return all_commands

        commands = check_all_commands_listed()
        max_len = max((len(c) for c in commands)) + 1
        fmt = '  %-{}s'.format(max_len)

        for group_name, comm_names in grps:
            self._out.writeln(group_name, Color.BRIGHT_MAGENTA)
            for name in comm_names:
                # future-proof way to ensure tabular formatting
                self._out.write(fmt % name, Color.GREEN)

                # Help will be all the lines up to the first empty one
                docstring_lines = commands[name].__doc__.split('\n')
                start = False
                data = []
                for line in docstring_lines:
                    line = line.strip()
                    if not line:
                        if start:
                            break
                        start = True
                        continue
                    data.append(line)

                import textwrap
                txt = textwrap.fill(' '.join(data), 80, subsequent_indent=" "*(max_len+2))
                self._out.writeln(txt)

        self._out.writeln("")
        self._out.writeln('Sense2 commands. Type "sense2 <command> -h" for help', Color.BRIGHT_YELLOW)   
    

def main(args):
    """ main entry point of the conan application, using a Command to
    parse parameters

    Exit codes for conan command:

        0: Success (done)
        1: General ConanException error (done)
        2: Migration error
        3: Ctrl+C
        4: Ctrl+Break
        5: SIGTERM
        6: Invalid configuration (done)
    """
    try:
        sense2_api, _, _ = Sense2.factory()
    except ConanMigrationError:  # Error migrating
        sys.exit(ERROR_MIGRATION)
    except ConanException as e:
        sys.stderr.write("Error in Conan initialization: {}".format(e))
        sys.exit(ERROR_GENERAL)

    def ctrl_c_handler(_, __):
        print('You pressed Ctrl+C!')
        sys.exit(USER_CTRL_C)

    def sigterm_handler(_, __):
        print('Received SIGTERM!')
        sys.exit(ERROR_SIGTERM)

    def ctrl_break_handler(_, __):
        print('You pressed Ctrl+Break!')
        sys.exit(USER_CTRL_BREAK)

    signal.signal(signal.SIGINT, ctrl_c_handler)
    signal.signal(signal.SIGTERM, sigterm_handler)

    if sys.platform == 'win32':
        signal.signal(signal.SIGBREAK, ctrl_break_handler)

    command = Command(sense2_api)
    error = command.run(args)
    sys.exit(error)



import unittest

from conans.test.utils.tools import TestClient, GenConanfile


class TransitiveIdsTest(unittest.TestCase):

    def test_transitive_library(self):
        # https://github.com/conan-io/conan/issues/6450
        client = TestClient()
        client.run("config set general.default_package_id_mode=full_version_mode")
        client.save({"conanfile.py": GenConanfile()})
        client.run("create . liba/1.0@")
        client.run("create . liba/1.1@")
        client.save({"conanfile.py": GenConanfile().with_require("liba/1.0")})
        client.run("create . libb/1.0@")
        client.save({"conanfile.py": GenConanfile().with_require("libb/1.0")})
        client.run("create . libc/1.0@")
        client.save({"conanfile.py": GenConanfile().with_require("libc/1.0")
                                                   .with_require("liba/1.0")})
        client.run("create . libd/1.0@")
        # The consumer forces to depend on liba/2, instead of liba/1
        client.save({"conanfile.py": GenConanfile().with_require("libc/1.0")
                                                   .with_require("liba/1.1")})
        client.run("create . libd/1.0@", assert_error=True)
        # both B and C require a new binary
        self.assertIn("liba/1.1:5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9 - Cache", client.out)
        self.assertIn("libb/1.0:50b928d46d42051a461440161b017eb6d52e2dff - Missing", client.out)
        self.assertIn("libc/1.0:4599994f77257f88443590022af4754d5520b753 - Missing", client.out)
        self.assertIn("libd/1.0:39906c34335d9ad465711e847688c4a27894af0f - Build", client.out)

    def test_transitive_major_mode(self):
        # https://github.com/conan-io/conan/issues/6450
        # Test LibE->LibD->LibC->LibB->LibA
        # LibC declares that it only depends on major version changes of its upstream
        # So LibC package ID doesn't change, even if LibA changes
        # But LibD package ID changes, even if its direct dependency LibC doesn't
        client = TestClient()
        client.run("config set general.default_package_id_mode=full_version_mode")
        # LibA
        client.save({"conanfile.py": GenConanfile()})
        client.run("create . liba/1.0@")
        client.run("create . liba/1.1@")
        # libB -> LibA
        client.save({"conanfile.py": GenConanfile().with_require("liba/1.0")})
        client.run("create . libb/1.0@")
        # libC -> libB
        major_mode = "self.info.requires.major_mode()"
        client.save({"conanfile.py": GenConanfile().with_require("libb/1.0")
                                                   .with_package_id(major_mode)})
        client.run("create . libc/1.0@")
        # Check the LibC ref with RREV keeps the same
        self.assertIn("libc/1.0:3627c16569f55501cb7d6c5db2b4b00faec7caf6 - Build", client.out)
        # LibD -> LibC
        client.save({"conanfile.py": GenConanfile().with_require("libc/1.0")})
        client.run("create . libd/1.0@")
        # LibE -> LibD, LibA/2.0
        client.save({"conanfile.py": GenConanfile().with_require("libd/1.0")
                                                   .with_require("liba/1.1")})
        client.run("create . libe/1.0@", assert_error=True)
        self.assertIn("liba/1.1:5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9 - Cache", client.out)
        self.assertIn("libb/1.0:50b928d46d42051a461440161b017eb6d52e2dff - Missing", client.out)
        # Check the LibC ref with RREV keeps the same, it is in cache, not missing
        self.assertIn("libc/1.0:3627c16569f55501cb7d6c5db2b4b00faec7caf6 - Cache", client.out)
        # But LibD package ID changes and is missing, because it depends transitively on LibA
        self.assertIn("libd/1.0:39906c34335d9ad465711e847688c4a27894af0f - Missing", client.out)
        self.assertIn("libe/1.0:204261ad030cca3acf07c7a58b169e4257056ba1 - Build", client.out)

    def test_transitive_unrelated(self):
        # https://github.com/conan-io/conan/issues/6450
        client = TestClient()
        client.run("config set general.default_package_id_mode=full_version_mode")
        # LibA
        client.save({"conanfile.py": GenConanfile()})
        client.run("create . liba/1.0@")
        client.run("create . liba/2.0@")
        # libB -> LibA
        client.save({"conanfile.py": GenConanfile().with_require("liba/1.0")})
        client.run("create . libb/1.0@")
        # libC -> libB
        unrelated = "self.info.requires['libb'].unrelated_mode()"
        client.save({"conanfile.py": GenConanfile().with_require("libb/1.0")
                    .with_package_id(unrelated)})
        client.run("create . libc/1.0@")
        # LibD -> LibC
        client.save({"conanfile.py": GenConanfile().with_require("libc/1.0")})
        client.run("create . libd/1.0@")
        # LibE -> LibD, LibA/2.0
        client.save({"conanfile.py": GenConanfile().with_require("libd/1.0")
                    .with_require("liba/2.0")})
        client.run("create . libe/1.0@", assert_error=True)
        self.assertIn("liba/2.0:5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9 - Cache", client.out)
        self.assertIn("libb/1.0:e71235a6f57633221a2b85f9b6aca14cda69e1fd - Missing", client.out)
        self.assertIn("libc/1.0:e3884c6976eb7debb8ec57aada7c0c2beaabe8ac - Missing", client.out)
        self.assertIn("libd/1.0:9b0b7b0905c9bc2cb9b7329f842b3b7c6663e8c3 - Missing", client.out)

    def test_transitive_second_level_header_only(self):
        # https://github.com/conan-io/conan/issues/6450
        client = TestClient()
        client.run("config set general.default_package_id_mode=full_version_mode")
        # LibA
        client.save({"conanfile.py": GenConanfile()})
        client.run("create . liba/1.0@")
        client.run("create . liba/2.0@")
        # libB -> LibA
        client.save({"conanfile.py": GenConanfile().with_require("liba/1.0")})
        client.run("create . libb/1.0@")
        # libC -> libB

        unrelated = "self.info.clear()"
        client.save({"conanfile.py": GenConanfile().with_require("libb/1.0")
                                                   .with_package_id(unrelated)})
        client.run("create . libc/1.0@")
        # LibD -> LibC
        client.save({"conanfile.py": GenConanfile().with_require("libc/1.0")})
        client.run("create . libd/1.0@")
        self.assertIn("libc/1.0:5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9 - Cache", client.out)

        # LibE -> LibD, LibA/2.0
        client.save({"conanfile.py": GenConanfile().with_require("libd/1.0")
                                                   .with_require("liba/2.0")})
        client.run("create . libe/1.0@", assert_error=True)  # LibD is NOT missing!
        self.assertIn("libd/1.0:119e0b2903330cef59977f8976cb82a665b510c1 - Cache", client.out)
        # USE THE NEW FIXED PACKAGE_ID
        client.run("config set general.full_transitive_package_id=1")
        client.run("create . libe/1.0@", assert_error=True)
        self.assertIn("liba/2.0:5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9 - Cache", client.out)
        self.assertIn("libb/1.0:e71235a6f57633221a2b85f9b6aca14cda69e1fd - Missing", client.out)
        self.assertIn("libc/1.0:5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9 - Cache", client.out)
        self.assertIn("libd/1.0:95b14a919aa70f9a7e24afbf48d1101cff344a67 - Missing", client.out)

    def test_transitive_header_only(self):
        # https://github.com/conan-io/conan/issues/6450
        client = TestClient()
        client.run("config set general.default_package_id_mode=full_version_mode")
        client.save({"conanfile.py": GenConanfile()})
        client.run("create . liba/1.0@")
        client.run("create . liba/2.0@")
        client.save({"conanfile.py": GenConanfile().with_require("liba/1.0")
                                                   .with_package_id("self.info.header_only()")})
        client.run("create . libb/1.0@")
        client.save({"conanfile.py": GenConanfile().with_require("libb/1.0")})
        client.run("create . libc/1.0@")
        client.save({"conanfile.py": GenConanfile().with_require("libc/1.0")
                                                   .with_require("liba/1.0")})
        client.run("create . libd/1.0@")
        client.save({"conanfile.py": GenConanfile().with_require("libc/1.0")
                                                   .with_require("liba/2.0")})

        client.run("create . libd/1.0@")  # Doesn't complain it is missing a binary!
        self.assertIn(" libc/1.0:fd60a00caf13b07bfce8690315c9e953aafd664b - Cache", client.out)
        # USE THE NEW FIXED PACKAGE_ID
        client.run("config set general.full_transitive_package_id=1")
        client.run("create . libd/1.0@", assert_error=True)
        self.assertIn("liba/2.0:5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9 - Cache", client.out)
        self.assertIn("libb/1.0:5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9 - Cache", client.out)
        self.assertIn("libc/1.0:bfa6c8f046896806f65c8fe554bd57f235b101e8 - Missing", client.out)



import os
import platform

import pytest

from conan import tools
from conan.tools.files import mkdir
from conans.test.utils.test_files import temp_folder


@pytest.fixture
def folders():
    tmp = temp_folder()
    files = ["foo/var/file.txt"]
    outside_folder = temp_folder()
    symlinks = [
        (os.path.join(tmp, "foo/var/file.txt"), "foo/var/other/absolute.txt"),  # Absolute link
        (os.path.join(tmp, "foo/var"), "foo/var/other/other/myfolder"),  # Absolute link folder
        (os.path.join(tmp, "foo/var/file.txt"), "foo/absolute.txt"),  # Absolute link
        ("foo/var/file.txt", "foo/var/other/relative.txt"),  # Relative link
        ("missing.txt", "foo/var/other/broken.txt"),  # Broken link
        (outside_folder, "foo/var/other/absolute_outside"),  # Absolute folder outside the folder
        ("../../../../../outside", "foo/absolute_outside"),  # Relative folder outside the folder
    ]
    # Create the files and symlinks
    for path in files:
        mkdir(None, os.path.dirname(os.path.join(tmp, path)))
        with open(os.path.join(tmp, path), "w") as fl:
            fl.write("foo")

    for link_dst, linked_file in symlinks:
        mkdir(None, os.path.dirname(os.path.join(tmp, linked_file)))
        os.symlink(link_dst, os.path.join(tmp, linked_file))
    return tmp, outside_folder


@pytest.mark.skipif(platform.system() == "Windows", reason="Symlinks Not in Windows")
def test_absolute_to_relative_symlinks(folders):
    """If a symlink is absolute but relative to a file or folder that is contained in
    the base folder, we can make it relative"""

    folder, outside_folder = folders
    # Transform the absolute symlinks to relative
    tools.files.symlinks.absolute_to_relative_symlinks(None, folder)

    # Check the results
    linked_to = os.readlink(os.path.join(folder, "foo/var/other/absolute.txt")).replace("\\", "/")
    assert linked_to == "../file.txt"

    linked_to = os.readlink(os.path.join(folder, "foo/var/other/other/myfolder")).replace("\\", "/")
    assert linked_to == "../.."

    linked_to = os.readlink(os.path.join(folder, "foo/absolute.txt")).replace("\\", "/")
    assert linked_to == "var/file.txt"

    linked_to = os.readlink(os.path.join(folder, "foo/var/other/relative.txt")).replace("\\", "/")
    assert linked_to == "foo/var/file.txt"

    linked_to = os.readlink(os.path.join(folder, "foo/var/other/broken.txt"))
    assert linked_to == "missing.txt"

    linked_to = os.readlink(os.path.join(folder, "foo/var/other/absolute_outside"))
    assert linked_to == outside_folder


@pytest.mark.skipif(platform.system() == "Windows", reason="Symlinks Not in Windows")
def test_remove_external_symlinks(folders):

    folder, outside_folder = folders
    # Remove the external symlinks
    tools.files.symlinks.remove_external_symlinks(None, folder)

    # Check the results, these are kept the same
    linked_to = os.readlink(os.path.join(folder, "foo/var/other/absolute.txt"))
    assert linked_to == os.path.join(folder, "foo/var/file.txt")

    linked_to = os.readlink(os.path.join(folder, "foo/var/other/other/myfolder"))
    assert linked_to == os.path.join(folder, "foo/var")

    linked_to = os.readlink(os.path.join(folder, "foo/absolute.txt"))
    assert linked_to == os.path.join(folder, "foo/var/file.txt")

    linked_to = os.readlink(os.path.join(folder, "foo/var/other/relative.txt"))
    assert linked_to == "foo/var/file.txt"

    linked_to = os.readlink(os.path.join(folder, "foo/var/other/broken.txt"))
    assert linked_to == "missing.txt"

    # This one is removed
    assert not os.path.islink(os.path.join(folder, "foo/var/other/absolute_outside"))
    assert not os.path.exists(os.path.join(folder, "foo/var/other/absolute_outside"))

    # This one is removed
    assert not os.path.islink(os.path.join(folder, "foo/absolute_outside"))
    assert not os.path.exists(os.path.join(folder, "foo/absolute_outside"))


@pytest.mark.skipif(platform.system() == "Windows", reason="Symlinks Not in Windows")
def test_remove_broken_symlinks(folders):
    folder, outside_folder = folders
    # Remove the external symlinks
    tools.files.symlinks.remove_broken_symlinks(None, folder)

    # Check the results, these are kept the same
    linked_to = os.readlink(os.path.join(folder, "foo/var/other/absolute.txt"))
    assert linked_to == os.path.join(folder, "foo/var/file.txt")

    linked_to = os.readlink(os.path.join(folder, "foo/var/other/other/myfolder"))
    assert linked_to == os.path.join(folder, "foo/var")

    linked_to = os.readlink(os.path.join(folder, "foo/absolute.txt"))
    assert linked_to == os.path.join(folder, "foo/var/file.txt")

    linked_to = os.readlink(os.path.join(folder, "foo/var/other/relative.txt"))
    assert linked_to == "foo/var/file.txt"

    # This one is removed
    assert not os.path.islink(os.path.join(folder, "foo/var/other/broken.txt"))
    assert not os.path.exists(os.path.join(folder, "foo/var/other/broken.txt"))

    linked_to = os.readlink(os.path.join(folder, "foo/var/other/absolute_outside"))
    assert linked_to == outside_folder

    # This is broken also so it is also removed
    assert not os.path.islink(os.path.join(folder, "foo/absolute_outside"))
    assert not os.path.exists(os.path.join(folder, "foo/absolute_outside"))



import os
import platform
import unittest

import pytest

from conans.model.ref import ConanFileReference
from conans.test.utils.tools import TestClient, GenConanfile


class RMdirFailTest(unittest.TestCase):

    @pytest.mark.skipif(platform.system() != "Windows", reason="needs windows")
    def test_fail_rmdir(self):
        client = TestClient()
        client.save({"conanfile.py": GenConanfile()})
        client.run("create . MyPkg/0.1@lasote/testing")
        ref = ConanFileReference.loads("MyPkg/0.1@lasote/testing")
        builds = client.cache.package_layout(ref).builds()
        build_folder = os.listdir(builds)[0]
        build_folder = os.path.join(builds, build_folder)
        f = open(os.path.join(build_folder, "myfile"), "wb")
        f.write(b"Hello world")
        client.run("install MyPkg/0.1@lasote/testing --build", assert_error=True)
        self.assertIn("Couldn't remove folder, might be busy or open", client.out)



import textwrap

from conans.test.utils.tools import TestClient


def test_basic():
    client = TestClient()
    conanfile = textwrap.dedent("""
        from conans import ConanFile

        class Pkg(ConanFile):

            def package_info(self):
                self.conf_info["tools.android:ndk_path"] = "MY-NDK!!!"
        """)
    client.save({"conanfile.py": conanfile})
    client.run("create . android_ndk/1.0@")

    consumer = textwrap.dedent("""
        from conans import ConanFile

        class Pkg(ConanFile):
            settings = "os", "compiler", "build_type", "arch"
            generators = "CMakeToolchain"
            build_requires = "android_ndk/1.0"

            def generate(self):
                self.output.info("NDK: %s" % self.conf["tools.android:ndk_path"])
        """)
    android_profile = textwrap.dedent("""
        include(default)
        [conf]
        tools.android:ndk_path=MY-SYSTEM-NDK!!!
        """)
    client.save({"conanfile.py": consumer,
                 "android": android_profile}, clean_first=True)
    client.run("install . -pr:b=default")
    assert "conanfile.py: NDK: MY-NDK!!!" in client.out

    client.run("install . -pr:b=default -pr:h=android")
    assert "conanfile.py: NDK: MY-SYSTEM-NDK!!!" in client.out


def test_basic_conf_through_cli():
    client = TestClient()
    conanfile = textwrap.dedent("""
        from conans import ConanFile

        class Pkg(ConanFile):

            def package_info(self):
                self.output.info("NDK build: %s" % self.conf["tools.android:ndk_path"])
        """)
    client.save({"conanfile.py": conanfile})
    client.run("create . android_ndk/1.0@")

    consumer = textwrap.dedent("""
        from conans import ConanFile

        class Pkg(ConanFile):
            settings = "os", "compiler", "build_type", "arch"
            generators = "CMakeToolchain"
            build_requires = "android_ndk/1.0"

            def generate(self):
                self.output.info("NDK host: %s" % self.conf["tools.android:ndk_path"])
        """)
    client.save({"conanfile.py": consumer}, clean_first=True)
    client.run('install . -c:b=tools.android:ndk_path="MY-NDK!!!" '
               '-c:h=tools.android:ndk_path="MY-SYSTEM-NDK!!!"')
    assert "android_ndk/1.0: NDK build: MY-NDK!!!" in client.out
    assert "conanfile.py: NDK host: MY-SYSTEM-NDK!!!" in client.out


def test_declared_generators_get_conf():
    # https://github.com/conan-io/conan/issues/9571
    client = TestClient()
    conanfile = textwrap.dedent("""
        from conans import ConanFile
        class Pkg(ConanFile):
            def package_info(self):
                self.conf_info.append("tools.cmake.cmaketoolchain:user_toolchain", "mytoolchain.cmake")
        """)
    client.save({"conanfile.py": conanfile})
    client.run("create . mytool/1.0@")

    consumer = textwrap.dedent("""
        from conans import ConanFile

        class Pkg(ConanFile):
            settings = "os", "compiler", "build_type", "arch"
            generators = "CMakeToolchain"
            build_requires = "mytool/1.0"
        """)
    client.save({"conanfile.py": consumer}, clean_first=True)
    client.run("install . -pr:b=default")
    toolchain = client.load("conan_toolchain.cmake")
    assert 'include("mytoolchain.cmake")' in toolchain

    consumer = textwrap.dedent("""
        from conans import ConanFile
        from conan.tools.cmake import CMakeToolchain

        class Pkg(ConanFile):
            settings = "os", "compiler", "build_type", "arch"
            build_requires = "mytool/1.0"

            def generate(self):
                CMakeToolchain(self).generate()
        """)
    client.save({"conanfile.py": consumer}, clean_first=True)
    client.run("install . -pr:b=default")
    toolchain = client.load("conan_toolchain.cmake")
    assert 'include("mytoolchain.cmake")' in toolchain



import platform
import textwrap
import unittest

from conans.test.utils.conan_v2_tests import ConanV2ModeTestCase
from conans.test.utils.tools import TestClient


class ConanfileSourceTestCase(ConanV2ModeTestCase):
    """ Conan v2: 'self.cpp_info' is not available in 'package_id()' """

    def test_cppinfo_not_in_package_id(self):
        # self.cpp_info is not available in 'package_id'
        t = self.get_client()
        conanfile = textwrap.dedent("""
            from conans import ConanFile

            class Recipe(ConanFile):

                def package_id(self):
                    self.cpp_info.libs = ["A"]
        """)
        t.save({'conanfile.py': conanfile})
        t.run('create . name/version@ -s os=Linux', assert_error=True)
        self.assertIn("Conan v2 incompatible: 'self.cpp_info' access in package_id() method is deprecated", t.out)


class ConanfilePackageIdV1TestCase(unittest.TestCase):
    """ Conan v1 will show a warning """

    def test_v1_warning(self):
        t = TestClient()
        conanfile = textwrap.dedent("""
            from conans import ConanFile

            class Recipe(ConanFile):

                def package_id(self):
                    self.cpp_info.libs = ["A"]  # No sense, it will warn the user
        """)
        t.save({'conanfile.py': conanfile})
        t.run('create . name/version@', assert_error=True)  # It is already raising
        self.assertIn("AttributeError: 'NoneType' object has no attribute 'libs'", t.out)
        # self.assertIn("name/version: WARN: 'self.info' access in package() method is deprecated", t.out)



import json
import os

from conans.model.graph_lock import GraphLockFile
from conans.test.utils.tools import TestClient, GenConanfile


def test_basic():
    client = TestClient()
    # TODO: This is hardcoded
    client.run("config set general.revisions_enabled=1")
    client.save({"pkga/conanfile.py": GenConanfile().with_settings("os"),
                 "pkgb/conanfile.py": GenConanfile().with_requires("pkga/0.1"),
                 "app1/conanfile.py": GenConanfile().with_settings("os").with_requires("pkgb/0.1"),
                 "app2/conanfile.py": GenConanfile().with_settings("os").with_requires("pkgb/0.2")})
    client.run("export pkga pkga/0.1@")
    client.run("export pkgb pkgb/0.1@")
    client.run("export pkgb pkgb/0.2@")
    client.run("export app1 app1/0.1@")
    client.run("export app2 app2/0.1@")
    client.run("lock create --ref=app1/0.1 --base --lockfile-out=app1_base.lock")
    client.run("lock create --ref=app2/0.1 --base --lockfile-out=app2_base.lock")

    client.run("lock create --ref=app1/0.1 -s os=Windows --lockfile=app1_base.lock "
               "--lockfile-out=app1_windows.lock")
    assert "app1/0.1:3bcd6800847f779e0883ee91b411aad9ddd8e83c - Missing" in client.out
    assert "pkga/0.1:3475bd55b91ae904ac96fde0f106a136ab951a5e - Missing" in client.out
    assert "pkgb/0.1:cfd10f60aeaa00f5ca1f90b5fe97c3fe19e7ec23 - Missing" in client.out
    client.run("lock create --ref=app1/0.1 -s os=Linux  --lockfile=app1_base.lock "
               "--lockfile-out=app1_linux.lock")
    assert "app1/0.1:60fbb0a22359b4888f7ecad69bcdfcd6e70e2784 - Missing" in client.out
    assert "pkga/0.1:cb054d0b3e1ca595dc66bc2339d40f1f8f04ab31 - Missing" in client.out
    assert "pkgb/0.1:cfd10f60aeaa00f5ca1f90b5fe97c3fe19e7ec23 - Missing" in client.out
    client.run("lock create --ref=app2/0.1 -s os=Windows  --lockfile=app2_base.lock "
               "--lockfile-out=app2_windows.lock")
    assert "app2/0.1:0f886d82040d47739aa363db84eef5fe4c958c23 - Missing" in client.out
    assert "pkga/0.1:3475bd55b91ae904ac96fde0f106a136ab951a5e - Missing" in client.out
    assert "pkgb/0.2:cfd10f60aeaa00f5ca1f90b5fe97c3fe19e7ec23 - Missing" in client.out
    client.run("lock create --ref=app2/0.1 -s os=Linux  --lockfile=app2_base.lock "
               "--lockfile-out=app2_linux.lock")
    assert "app2/0.1:156f38906bdcdceba1b26a206240cf199619fee1 - Missing" in client.out
    assert "pkga/0.1:cb054d0b3e1ca595dc66bc2339d40f1f8f04ab31 - Missing" in client.out
    assert "pkgb/0.2:cfd10f60aeaa00f5ca1f90b5fe97c3fe19e7ec23 - Missing" in client.out

    client.run("lock bundle create app1_windows.lock app1_linux.lock "
               "app2_windows.lock app2_linux.lock --bundle-out=lock1.bundle")

    client.run("lock bundle build-order lock1.bundle --json=bo.json")
    order = client.load("bo.json")
    order = json.loads(order)
    assert order == [
        ["pkga/0.1@#f096d7d54098b7ad7012f9435d9c33f3"],
        ["pkgb/0.1@#cd8f22d6f264f65398d8c534046e8e20", "pkgb/0.2@#cd8f22d6f264f65398d8c534046e8e20"],
        ["app1/0.1@#584778f98ba1d0eb7c80a5ae1fe12fe2", "app2/0.1@#3850895c1eac8223c43c71d525348019"]
    ]
    bundle = client.load("lock1.bundle")
    bundle = json.loads(bundle)["lock_bundle"]
    for level in order:
        for ref in level:
            # Now get the package_id, lockfile
            packages = bundle[ref]["packages"]
            for pkg in packages:
                lockfiles = pkg["lockfiles"]
                lockfile = next(iter(sorted(lockfiles)))

                client.run("install {ref} --build={ref} --lockfile={lockfile} "
                           "--lockfile-out={lockfile}".format(ref=ref, lockfile=lockfile))
                client.run("lock bundle update lock1.bundle")

    app1_win = GraphLockFile.load(os.path.join(client.current_folder, "app1_windows.lock"),
                                  client.cache.config.revisions_enabled)
    nodes = app1_win.graph_lock.nodes
    assert nodes["1"].modified is True
    assert nodes["1"].ref.full_str() == "app1/0.1#584778f98ba1d0eb7c80a5ae1fe12fe2"
    assert nodes["1"].package_id == "3bcd6800847f779e0883ee91b411aad9ddd8e83c"
    assert nodes["1"].prev == "c6658a5c66393cf4d210c35b5fbf34f8"

    assert nodes["2"].modified is True
    assert nodes["2"].ref.full_str() == "pkgb/0.1#cd8f22d6f264f65398d8c534046e8e20"
    assert nodes["2"].package_id == "cfd10f60aeaa00f5ca1f90b5fe97c3fe19e7ec23"
    assert nodes["2"].prev == "d61b9f421cada3b4d8e39540b0aea3d0"

    assert nodes["3"].modified is True
    assert nodes["3"].ref.full_str() == "pkga/0.1#f096d7d54098b7ad7012f9435d9c33f3"
    assert nodes["3"].package_id == "3475bd55b91ae904ac96fde0f106a136ab951a5e"
    assert nodes["3"].prev == "d0f0357277b3417d3984b5a9a85bbab6"

    app2_linux = GraphLockFile.load(os.path.join(client.current_folder, "app2_linux.lock"),
                                    client.cache.config.revisions_enabled)
    nodes = app2_linux.graph_lock.nodes
    assert nodes["1"].modified is True
    assert nodes["1"].ref.full_str() == "app2/0.1#3850895c1eac8223c43c71d525348019"
    assert nodes["1"].package_id == "156f38906bdcdceba1b26a206240cf199619fee1"
    assert nodes["1"].prev == "f1ca88c668b7f573037d09cb04be0e6f"

    assert nodes["2"].modified is True
    assert nodes["2"].ref.full_str() == "pkgb/0.2#cd8f22d6f264f65398d8c534046e8e20"
    assert nodes["2"].package_id == "cfd10f60aeaa00f5ca1f90b5fe97c3fe19e7ec23"
    assert nodes["2"].prev == "d61b9f421cada3b4d8e39540b0aea3d0"

    assert nodes["3"].modified is True
    assert nodes["3"].ref.full_str() == "pkga/0.1#f096d7d54098b7ad7012f9435d9c33f3"
    assert nodes["3"].package_id == "cb054d0b3e1ca595dc66bc2339d40f1f8f04ab31"
    assert nodes["3"].prev == "9e99cfd92d0d7df79d687b01512ce844"

    client.run("lock bundle clean-modified lock1.bundle")
    bundle = client.load("lock1.bundle")
    assert '"modified": true' not in bundle
    lock1 = client.load("app1_windows.lock")
    assert '"modified": true' not in lock1
    lock2 = client.load("app2_linux.lock")
    assert '"modified": true' not in lock2


def test_build_requires():
    client = TestClient()
    # TODO: This is hardcoded
    client.run("config set general.revisions_enabled=1")
    client.save({"tool/conanfile.py": GenConanfile().with_settings("os"),
                 "pkga/conanfile.py": GenConanfile().with_settings("os"),
                 "pkgb/conanfile.py": GenConanfile().with_requires("pkga/0.1"),
                 "app1/conanfile.py": GenConanfile().with_settings("os").with_requires("pkgb/0.1"),
                 "app2/conanfile.py": GenConanfile().with_settings("os").with_requires("pkgb/0.2"),
                 "profile": "[build_requires]\ntool/0.1"})
    client.run("export tool tool/0.1@")
    client.run("export pkga pkga/0.1@")
    client.run("export pkgb pkgb/0.1@")
    client.run("export pkgb pkgb/0.2@")
    client.run("export app1 app1/0.1@")
    client.run("export app2 app2/0.1@")
    client.run("lock create --ref=app1/0.1 --base --lockfile-out=app1_base.lock")
    client.run("lock create --ref=app2/0.1 --base --lockfile-out=app2_base.lock")

    client.run("lock create --ref=app1/0.1 -pr=profile -s os=Windows --lockfile=app1_base.lock "
               "--lockfile-out=app1_windows.lock --build=missing")
    assert "tool/0.1:3475bd55b91ae904ac96fde0f106a136ab951a5e - Build" in client.out
    assert "app1/0.1:3bcd6800847f779e0883ee91b411aad9ddd8e83c - Build" in client.out
    assert "pkga/0.1:3475bd55b91ae904ac96fde0f106a136ab951a5e - Build" in client.out
    assert "pkgb/0.1:cfd10f60aeaa00f5ca1f90b5fe97c3fe19e7ec23 - Build" in client.out
    client.run("lock create --ref=app1/0.1 -pr=profile -s os=Linux  --lockfile=app1_base.lock "
               "--lockfile-out=app1_linux.lock --build=missing")
    assert "tool/0.1:cb054d0b3e1ca595dc66bc2339d40f1f8f04ab31 - Build" in client.out
    assert "app1/0.1:60fbb0a22359b4888f7ecad69bcdfcd6e70e2784 - Build" in client.out
    assert "pkga/0.1:cb054d0b3e1ca595dc66bc2339d40f1f8f04ab31 - Build" in client.out
    assert "pkgb/0.1:cfd10f60aeaa00f5ca1f90b5fe97c3fe19e7ec23 - Build" in client.out
    client.run("lock create --ref=app2/0.1 -pr=profile -s os=Windows  --lockfile=app2_base.lock "
               "--lockfile-out=app2_windows.lock --build=missing")
    assert "tool/0.1:3475bd55b91ae904ac96fde0f106a136ab951a5e - Build" in client.out
    assert "app2/0.1:0f886d82040d47739aa363db84eef5fe4c958c23 - Build" in client.out
    assert "pkga/0.1:3475bd55b91ae904ac96fde0f106a136ab951a5e - Build" in client.out
    assert "pkgb/0.2:cfd10f60aeaa00f5ca1f90b5fe97c3fe19e7ec23 - Build" in client.out
    client.run("lock create --ref=app2/0.1 -pr=profile -s os=Linux  --lockfile=app2_base.lock "
               "--lockfile-out=app2_linux.lock --build=missing")
    assert "tool/0.1:cb054d0b3e1ca595dc66bc2339d40f1f8f04ab31 - Build" in client.out
    assert "app2/0.1:156f38906bdcdceba1b26a206240cf199619fee1 - Build" in client.out
    assert "pkga/0.1:cb054d0b3e1ca595dc66bc2339d40f1f8f04ab31 - Build" in client.out
    assert "pkgb/0.2:cfd10f60aeaa00f5ca1f90b5fe97c3fe19e7ec23 - Build" in client.out

    client.run("lock bundle create app1_windows.lock app1_linux.lock "
               "app2_windows.lock app2_linux.lock --bundle-out=lock1.bundle")

    client.run("lock bundle build-order lock1.bundle --json=bo.json")
    order = client.load("bo.json")
    order = json.loads(order)
    assert order == [
        ["tool/0.1@#f096d7d54098b7ad7012f9435d9c33f3"],
        ["pkga/0.1@#f096d7d54098b7ad7012f9435d9c33f3"],
        ["pkgb/0.1@#cd8f22d6f264f65398d8c534046e8e20", "pkgb/0.2@#cd8f22d6f264f65398d8c534046e8e20"],
        ["app1/0.1@#584778f98ba1d0eb7c80a5ae1fe12fe2", "app2/0.1@#3850895c1eac8223c43c71d525348019"]
    ]
    bundle = client.load("lock1.bundle")
    print(bundle)
    bundle = json.loads(bundle)["lock_bundle"]
    for level in order:
        for ref in level:
            # Now get the package_id, lockfile
            packages = bundle[ref]["packages"]
            for pkg in packages:
                lockfiles = pkg["lockfiles"]
                lockfile = next(iter(sorted(lockfiles)))

                client.run("install {ref} --build={ref} --lockfile={lockfile} "
                           "--lockfile-out={lockfile}".format(ref=ref, lockfile=lockfile))
                client.run("lock bundle update lock1.bundle")

    app1_win = GraphLockFile.load(os.path.join(client.current_folder, "app1_windows.lock"),
                                  client.cache.config.revisions_enabled)
    nodes = app1_win.graph_lock.nodes
    assert nodes["1"].modified is True
    assert nodes["1"].ref.full_str() == "app1/0.1#584778f98ba1d0eb7c80a5ae1fe12fe2"
    assert nodes["1"].package_id == "3bcd6800847f779e0883ee91b411aad9ddd8e83c"
    assert nodes["1"].prev == "c6658a5c66393cf4d210c35b5fbf34f8"

    assert nodes["2"].modified is True
    assert nodes["2"].ref.full_str() == "pkgb/0.1#cd8f22d6f264f65398d8c534046e8e20"
    assert nodes["2"].package_id == "cfd10f60aeaa00f5ca1f90b5fe97c3fe19e7ec23"
    assert nodes["2"].prev == "d61b9f421cada3b4d8e39540b0aea3d0"

    assert nodes["3"].modified is True
    assert nodes["3"].ref.full_str() == "pkga/0.1#f096d7d54098b7ad7012f9435d9c33f3"
    assert nodes["3"].package_id == "3475bd55b91ae904ac96fde0f106a136ab951a5e"
    assert nodes["3"].prev == "d0f0357277b3417d3984b5a9a85bbab6"

    for n in ("5", "6", "7"):
        assert nodes[n].modified is True
        assert nodes[n].ref.full_str() == "tool/0.1#f096d7d54098b7ad7012f9435d9c33f3"
        assert nodes[n].package_id == "3475bd55b91ae904ac96fde0f106a136ab951a5e"
        assert nodes[n].prev == "d0f0357277b3417d3984b5a9a85bbab6"

    app2_linux = GraphLockFile.load(os.path.join(client.current_folder, "app2_linux.lock"),
                                    client.cache.config.revisions_enabled)
    nodes = app2_linux.graph_lock.nodes
    assert nodes["1"].modified is True
    assert nodes["1"].ref.full_str() == "app2/0.1#3850895c1eac8223c43c71d525348019"
    assert nodes["1"].package_id == "156f38906bdcdceba1b26a206240cf199619fee1"
    assert nodes["1"].prev == "f1ca88c668b7f573037d09cb04be0e6f"

    assert nodes["2"].modified is True
    assert nodes["2"].ref.full_str() == "pkgb/0.2#cd8f22d6f264f65398d8c534046e8e20"
    assert nodes["2"].package_id == "cfd10f60aeaa00f5ca1f90b5fe97c3fe19e7ec23"
    assert nodes["2"].prev == "d61b9f421cada3b4d8e39540b0aea3d0"

    assert nodes["3"].modified is True
    assert nodes["3"].ref.full_str() == "pkga/0.1#f096d7d54098b7ad7012f9435d9c33f3"
    assert nodes["3"].package_id == "cb054d0b3e1ca595dc66bc2339d40f1f8f04ab31"
    assert nodes["3"].prev == "9e99cfd92d0d7df79d687b01512ce844"

    for n in ("5", "6", "7"):
        assert nodes[n].modified is True
        assert nodes[n].ref.full_str() == "tool/0.1#f096d7d54098b7ad7012f9435d9c33f3"
        assert nodes[n].package_id == "cb054d0b3e1ca595dc66bc2339d40f1f8f04ab31"
        assert nodes[n].prev == "9e99cfd92d0d7df79d687b01512ce844"


def test_build_requires_error():
    # https://github.com/conan-io/conan/issues/8577
    client = TestClient()
    # TODO: This is hardcoded
    client.run("config set general.revisions_enabled=1")
    client.save({"tool/conanfile.py": GenConanfile().with_settings("os"),
                 "pkga/conanfile.py": GenConanfile().with_settings("os"),
                 "app1/conanfile.py": GenConanfile().with_settings("os").with_requires("pkga/0.1"),
                 "profile": "[build_requires]\ntool/0.1"})
    client.run("create tool tool/0.1@ -s os=Windows")
    client.run("create tool tool/0.1@ -s os=Linux")
    client.run("export pkga pkga/0.1@")
    client.run("export app1 app1/0.1@")

    client.run("lock create --ref=app1/0.1 -pr=profile -s os=Windows "
               "--lockfile-out=app1_windows.lock --build=missing")
    assert "tool/0.1:3475bd55b91ae904ac96fde0f106a136ab951a5e - Cache" in client.out
    client.run("lock create --ref=app1/0.1 -pr=profile -s os=Linux "
               "--lockfile-out=app1_linux.lock --build=missing")
    assert "tool/0.1:cb054d0b3e1ca595dc66bc2339d40f1f8f04ab31 - Cache" in client.out

    client.run("lock bundle create app1_windows.lock app1_linux.lock --bundle-out=lock1.bundle")
    client.run("lock bundle build-order lock1.bundle --json=bo.json")
    order = client.load("bo.json")
    print(order)
    order = json.loads(order)
    assert order == [
        ["pkga/0.1@#f096d7d54098b7ad7012f9435d9c33f3"],
        ["app1/0.1@#5af607abc205b47375f485a98abc3b38"]
    ]



import os
import platform
import shutil
import textwrap
import unittest

import pytest
from parameterized import parameterized

from conans.client.tools import environment_append
from conans.model.ref import ConanFileReference, PackageReference
from conans.test.assets.genconanfile import GenConanfile
from conans.test.utils.test_files import temp_folder
from conans.test.utils.tools import NO_SETTINGS_PACKAGE_ID, TestClient


@pytest.mark.skipif(platform.system() != "Windows", reason="Requires Windows")
class ShortPathsTest(unittest.TestCase):

    def test_inconsistent_cache(self):
        conanfile = """
import os
from conans import ConanFile, tools


class TestConan(ConanFile):
    name = "test"
    version = "1.0"
    short_paths = {0}
    exports_sources = "source_file.cpp"

    def source(self):
        for item in os.listdir(self.source_folder):
            self.output.info("SOURCE: " + str(item))
    def build(self):
        tools.save(os.path.join(self.build_folder, "artifact"), "")
        for item in os.listdir(self.build_folder):
            self.output.info("BUILD: " + str(item))
    def package(self):
        self.copy("source_file.cpp")
        self.copy("artifact")
        for item in os.listdir(self.build_folder):
            self.output.info("PACKAGE: " + str(item))
"""

        client = TestClient()
        client.save({"conanfile.py": conanfile.format("False"),
                     "source_file.cpp": ""})
        client.run("create . danimtb/testing")
        ref = ConanFileReference("test", "1.0", "danimtb", "testing")
        source_folder = os.path.join(client.cache.package_layout(ref).base_folder(), "source")
        build_folder = os.path.join(client.cache.package_layout(ref).base_folder(), "build",
                                    NO_SETTINGS_PACKAGE_ID)
        package_folder = os.path.join(client.cache.package_layout(ref).base_folder(), "package",
                                      NO_SETTINGS_PACKAGE_ID)
        self.assertIn("SOURCE: source_file.cpp", client.out)
        self.assertEqual(["source_file.cpp"], os.listdir(source_folder))
        self.assertIn("BUILD: source_file.cpp", client.out)
        self.assertIn("BUILD: artifact", client.out)
        self.assertEqual(
            sorted(["artifact", "conanbuildinfo.txt", "conaninfo.txt", "source_file.cpp"]),
            sorted(os.listdir(build_folder)))
        self.assertIn("PACKAGE: source_file.cpp", client.out)
        self.assertIn("PACKAGE: artifact", client.out)
        self.assertEqual(
            sorted(["artifact", "conaninfo.txt", "conanmanifest.txt", "source_file.cpp"]),
            sorted(os.listdir(package_folder)))
        client.save({"conanfile.py": conanfile.format("True")})
        client.run("create . danimtb/testing")
        self.assertIn("SOURCE: source_file.cpp", client.out)
        self.assertEqual([".conan_link"], os.listdir(source_folder))
        self.assertIn("BUILD: source_file.cpp", client.out)
        self.assertIn("BUILD: artifact", client.out)
        self.assertEqual([".conan_link"], os.listdir(build_folder))
        self.assertIn("PACKAGE: source_file.cpp", client.out)
        self.assertIn("PACKAGE: artifact", client.out)
        self.assertEqual([".conan_link"], os.listdir(package_folder))

    def test_package_output(self):
        conanfile = textwrap.dedent("""
            from conans import ConanFile

            class TestConan(ConanFile):
                name = "test"
                version = "1.0"
                short_paths = True
            """)
        client = TestClient()
        client.save({"conanfile.py": conanfile})
        client.run("create . danimtb/testing")
        self.assertNotIn("test/1.0@danimtb/testing: Package '1' created", client.out)
        self.assertIn("test/1.0@danimtb/testing: Package '%s' created" % NO_SETTINGS_PACKAGE_ID,
                      client.out)

        # try local flow still works, but no pkg id available
        client.run("install .")
        client.run("package .")
        self.assertIn("conanfile.py (test/1.0): Package 'package' created", client.out)

        # try export-pkg with package folder
        client.run("remove test/1.0@danimtb/testing --force")
        client.run("export-pkg . test/1.0@danimtb/testing --package-folder package")
        self.assertIn("test/1.0@danimtb/testing: Package '%s' created" % NO_SETTINGS_PACKAGE_ID,
                      client.out)

        # try export-pkg without package folder
        client.run("remove test/1.0@danimtb/testing --force")
        client.run("export-pkg . test/1.0@danimtb/testing --install-folder .")
        self.assertIn("test/1.0@danimtb/testing: Package '%s' created" % NO_SETTINGS_PACKAGE_ID,
                      client.out)

        # try conan get
        client.run("get test/1.0@danimtb/testing . -p %s" % NO_SETTINGS_PACKAGE_ID)
        self.assertIn("conaninfo.txt", client.out)
        self.assertIn("conanmanifest.txt", client.out)

    @parameterized.expand([(True,), (False,)])
    def test_leaking_folders(self, use_always_short_paths):
        # https://github.com/conan-io/conan/issues/7983
        client = TestClient(cache_autopopulate=False)
        short_folder = temp_folder()
        # Testing in CI requires setting it via env-var
        with environment_append({"CONAN_USER_HOME_SHORT": short_folder}):
            conanfile = GenConanfile().with_exports_sources("*")
            if use_always_short_paths:
                client.run("config set general.use_always_short_paths=True")
            else:
                conanfile.with_short_paths(True)
            client.save({"conanfile.py": conanfile,
                         "file.h": ""})
            client.run("create . dep/1.0@")
            self.assertEqual(5, len(os.listdir(short_folder)))
            client.run("remove * -f")
            self.assertEqual(0, len(os.listdir(short_folder)))
            client.run("create . dep/1.0@")
            self.assertEqual(5, len(os.listdir(short_folder)))
            client.run("install dep/1.0@")
            self.assertEqual(5, len(os.listdir(short_folder)))
            client.run("install dep/1.0@ --build")
            self.assertEqual(5, len(os.listdir(short_folder)))

    def test_info_paths(self):
        # https://github.com/conan-io/conan/issues/8172
        client = TestClient()
        client.save({"conanfile.py": GenConanfile().with_short_paths(True)})
        client.run("export . test/1.0@")
        client.run("info test/1.0@ --paths")
        client.run("info test/1.0@ --paths")



import os
import textwrap

import pytest

from conans.test.utils.tools import TestClient


@pytest.fixture(scope="module")
def hello_client():
    client = TestClient()
    client.run("new hello/1.1 --template=cmake_lib")
    client.run("create . -tf=None")
    return client


@pytest.mark.parametrize("name, version, params, cmake_fails, package_found", [
    ("hello", "1.0", "", False, True),
    ("Hello", "1.0", "", False, True),
    ("HELLO", "1.0", "", False, True),
    ("hello", "1.1", "", False, True),
    ("hello", "1.2", "", False, False),
    ("hello", "1.0", "EXACT", False, False),
    ("hello", "1.1", "EXACT", False, True),
    ("hello", "1.2", "EXACT", False, False),
    ("hello", "0.1", "", False, False),
    ("hello", "2.0", "", False, False),
    ("hello", "1.0", "REQUIRED", False, True),
    ("hello", "1.1", "REQUIRED", False, True),
    ("hello", "1.2", "REQUIRED", True, False),
    ("hello", "1.0", "EXACT REQUIRED", True, False),
    ("hello", "1.1", "EXACT REQUIRED", False, True),
    ("hello", "1.2", "EXACT REQUIRED", True, False),
    ("hello", "0.1", "REQUIRED", True, False),
    ("hello", "2.0", "REQUIRED", True, False)
])
def test_version(hello_client, name, version, params, cmake_fails, package_found):
    client = hello_client
    cmakelists = textwrap.dedent("""
        cmake_minimum_required(VERSION 3.1)
        project(consumer NONE)
        find_package({name} {version} {params})
        message(STATUS "hello found: ${{{name}_FOUND}}")
        """).format(name=name, version=version, params=params)

    conanfile = textwrap.dedent("""
        from conans import ConanFile
        from conan.tools.cmake import CMake

        class Conan(ConanFile):
            requires = "hello/1.1"
            settings = "os", "compiler", "arch", "build_type"
            generators = "CMakeDeps", "CMakeToolchain"

            def build(self):
                cmake = CMake(self)
                cmake.configure()
        """)

    client.save({"conanfile.py": conanfile, "CMakeLists.txt": cmakelists}, clean_first=True)
    client.run("install .")
    exit_code = client.run("build .", assert_error=cmake_fails)
    if cmake_fails:
        assert exit_code != 0
    elif package_found:
        assert "hello found: 1" in client.out
    else:
        assert "hello found: 0" in client.out


def test_no_version_file(hello_client):
    client = hello_client

    cmakelists = textwrap.dedent("""
        cmake_minimum_required(VERSION 3.1)
        project(consumer NONE)
        find_package(hello 1.0 REQUIRED)
        message(STATUS "hello found: ${{hello_FOUND}}")
        """)

    conanfile = textwrap.dedent("""
        from conans import ConanFile
        from conan.tools.cmake import CMake

        class Conan(ConanFile):
            settings = "os", "compiler", "arch", "build_type"
            requires = "hello/1.1"
            generators = "CMakeDeps", "CMakeToolchain"

            def build(self):
                cmake = CMake(self)
                cmake.configure()
        """)

    client.save({"conanfile.py": conanfile, "CMakeLists.txt": cmakelists}, clean_first=True)
    client.run("install .")
    os.unlink(os.path.join(client.current_folder, "hello-config-version.cmake"))
    exit_code = client.run("build .", assert_error=True)
    assert 0 != exit_code



import platform
import textwrap

import pytest

from conan.tools.files.files import load_toolchain_args
from conans.client.tools.apple import XCRun, to_apple_arch
from conans.test.assets.autotools import gen_makefile_am, gen_configure_ac
from conans.test.assets.sources import gen_function_cpp
from conans.test.utils.tools import TestClient


@pytest.mark.skipif(platform.system() != "Darwin", reason="Requires Xcode")
def test_ios():
    xcrun = XCRun(None, sdk='iphoneos')
    cflags = ""
    cflags += " -isysroot " + xcrun.sdk_path
    cflags += " -arch " + to_apple_arch('armv8')
    cxxflags = cflags
    ldflags = cflags

    profile = textwrap.dedent("""
        include(default)
        [settings]
        os=iOS
        os.sdk=iphoneos
        os.version=12.0
        arch=armv8
        [env]
        CC={cc}
        CXX={cxx}
        CFLAGS={cflags}
        CXXFLAGS={cxxflags}
        LDFLAGS={ldflags}
    """).format(cc=xcrun.cc, cxx=xcrun.cxx, cflags=cflags, cxxflags=cxxflags, ldflags=ldflags)

    client = TestClient(path_with_spaces=False)
    client.save({"m1": profile}, clean_first=True)
    client.run("new hello/0.1 --template=cmake_lib")
    client.run("create . --profile:build=default --profile:host=m1 -tf None")

    main = gen_function_cpp(name="main", includes=["hello"], calls=["hello"])
    makefile_am = gen_makefile_am(main="main", main_srcs="main.cpp")
    configure_ac = gen_configure_ac()

    conanfile = textwrap.dedent("""
        from conans import ConanFile
        from conan.tools.gnu import Autotools

        class TestConan(ConanFile):
            requires = "hello/0.1"
            settings = "os", "compiler", "arch", "build_type"
            exports_sources = "configure.ac", "Makefile.am", "main.cpp"
            generators = "AutotoolsToolchain", "AutotoolsDeps"

            def layout(self):
                self.cpp.package.resdirs = ["res"]

            def build(self):
                autotools = Autotools(self)
                autotools.autoreconf()
                autotools.configure()
                autotools.make()
        """)

    client.save({"conanfile.py": conanfile,
                 "configure.ac": configure_ac,
                 "Makefile.am": makefile_am,
                 "main.cpp": main,
                 "m1": profile}, clean_first=True)
    client.run("install . --profile:build=default --profile:host=m1")
    client.run("build .")
    client.run_command("lipo -info main")
    assert "Non-fat file: main is architecture: arm64" in client.out
    client.run_command("vtool -show-build main")
    assert "platform IOS" in client.out
    assert "minos 12.0" in client.out

    conanbuild = load_toolchain_args(client.current_folder)
    configure_args = conanbuild["configure_args"]
    make_args = conanbuild["make_args"]
    autoreconf_args = conanbuild["autoreconf_args"]
    build_arch = client.get_default_build_profile().settings['arch']
    build_arch = "aarch64" if build_arch == "armv8" else build_arch
    assert configure_args == "'--prefix=/' '--bindir=${prefix}/bin' '--sbindir=${prefix}/bin' " \
                             "'--libdir=${prefix}/lib' '--includedir=${prefix}/include' " \
                             "'--oldincludedir=${prefix}/include' '--datarootdir=${prefix}/res' " \
                             f"'--host=aarch64-apple-ios' '--build={build_arch}-apple-darwin'"
    assert make_args == ""
    assert autoreconf_args == "'--force' '--install'"



import zipfile

import os
import tarfile
import unittest

import six
from mock import Mock

from conans.client.tools import untargz, unzip
from conan.tools.files import unzip as unzip_dev2
from conans.client.tools.files import chdir, save
from conans.test.utils.mocks import TestBufferConanOutput, ConanFileMock
from conans.test.utils.test_files import temp_folder
from conans.errors import ConanException
from conans.model.manifest import gather_files
from conans.util.files import gzopen_without_timestamps, rmdir


class ZipExtractPlainTest(unittest.TestCase):

    def _zipdir(self, path, zipfilename, folder_entry=None):
        with zipfile.ZipFile(zipfilename, 'w', zipfile.ZIP_DEFLATED) as z:
            if folder_entry:
                zif = zipfile.ZipInfo(folder_entry + "/")
                z.writestr(zif, "")

            for root, _, files in os.walk(path):
                for f in files:
                    file_path = os.path.join(root, f)
                    if file_path == zipfilename:
                        continue
                    relpath = os.path.relpath(file_path, path)
                    z.write(file_path, relpath)

    def test_plain_zip(self):
        tmp_folder = temp_folder()
        with chdir(tmp_folder):
            ori_files_dir = os.path.join(tmp_folder, "subfolder-1.2.3")
            file1 = os.path.join(ori_files_dir, "file1")
            file2 = os.path.join(ori_files_dir, "folder", "file2")
            file3 = os.path.join(ori_files_dir, "file3")

            save(file1, "")
            save(file2, "")
            save(file3, "")

        zip_file = os.path.join(tmp_folder, "myzip.zip")
        # Zip with a "folder_entry" in the zip (not only for the files)
        self._zipdir(tmp_folder, zip_file, folder_entry="subfolder-1.2.3")

        # ZIP unzipped regularly
        extract_folder = temp_folder()
        output = TestBufferConanOutput()
        unzip(zip_file, destination=extract_folder, strip_root=False, output=output)
        self.assertNotIn("ERROR: Error extract", output)
        self.assertTrue(os.path.exists(os.path.join(extract_folder, "subfolder-1.2.3")))
        self.assertTrue(os.path.exists(os.path.join(extract_folder, "subfolder-1.2.3", "file1")))
        self.assertTrue(os.path.exists(os.path.join(extract_folder, "subfolder-1.2.3", "folder",
                                                    "file2")))
        self.assertTrue(os.path.exists(os.path.join(extract_folder, "subfolder-1.2.3", "file3")))

        # Extract without the subfolder
        extract_folder = temp_folder()
        output = TestBufferConanOutput()
        unzip(zip_file, destination=extract_folder, strip_root=True, output=output)
        self.assertNotIn("ERROR: Error extract", output)
        self.assertFalse(os.path.exists(os.path.join(extract_folder, "subfolder-1.2.3")))
        self.assertTrue(os.path.exists(os.path.join(extract_folder, "file1")))
        self.assertTrue(os.path.exists(os.path.join(extract_folder, "folder", "file2")))
        self.assertTrue(os.path.exists(os.path.join(extract_folder, "file3")))

    def test_invalid_flat(self):
        tmp_folder = temp_folder()
        with chdir(tmp_folder):
            # Not a single dir containing everything
            file1 = os.path.join(tmp_folder, "subfolder-1.2.3", "folder2", "file1")
            file2 = os.path.join(tmp_folder, "other-1.2.3", "folder", "file2")

            save(file1, "")
            save(file2, "")

        zip_folder = temp_folder()
        zip_file = os.path.join(zip_folder, "file.zip")
        self._zipdir(tmp_folder, zip_file)

        # Extract without the subfolder
        extract_folder = temp_folder()
        with six.assertRaisesRegex(self, ConanException, "The zip file contains more than 1 folder "
                                                         "in the root"):
            unzip(zip_file, destination=extract_folder, strip_root=True, output=Mock())

    def test_invalid_flat_single_file(self):
        tmp_folder = temp_folder()
        with chdir(tmp_folder):
            save("file1", "contentsfile1")

        zip_folder = temp_folder()
        zip_file = os.path.join(zip_folder, "file.zip")
        self._zipdir(tmp_folder, zip_file)

        # Extract without the subfolder
        extract_folder = temp_folder()
        with six.assertRaisesRegex(self, ConanException, "The zip file contains a file in the root"):
            unzip(zip_file, destination=extract_folder, strip_root=True, output=Mock())


class TarExtractPlainTest(unittest.TestCase):

    def _compress_folder(self, folder, tgz_path, folder_entry=None):
        # Create a tar.gz file with the files in the folder and an additional TarInfo entry
        # for the folder_entry (the gather files doesn't return empty dirs)
        with open(tgz_path, "wb") as tgz_handle:
            tgz = gzopen_without_timestamps("name", mode="w", fileobj=tgz_handle)
            if folder_entry:
                # Create an empty folder in the tgz file
                t = tarfile.TarInfo(folder_entry)
                t.mode = 488
                t.type = tarfile.DIRTYPE
                tgz.addfile(t)
            files, _ = gather_files(folder)
            for filename, abs_path in files.items():
                info = tarfile.TarInfo(name=filename)
                with open(os.path.join(folder, filename), 'rb') as file_handler:
                    tgz.addfile(tarinfo=info, fileobj=file_handler)
            tgz.close()

    def test_linkame_striproot_folder(self):
        tmp_folder = temp_folder()
        other_tmp_folder = temp_folder()
        save(os.path.join(other_tmp_folder, "foo.txt"), "")
        tgz_path = os.path.join(tmp_folder, "foo.tgz")

        with open(tgz_path, "wb") as tgz_handle:
            tgz = gzopen_without_timestamps("name", mode="w", fileobj=tgz_handle)

            # Regular file
            info = tarfile.TarInfo(name="common/foo.txt")
            info.name = "common/subfolder/foo.txt"
            info.path = "common/subfolder/foo.txt"
            with open(os.path.join(other_tmp_folder, "foo.txt"), 'rb') as file_handler:
                tgz.addfile(tarinfo=info, fileobj=file_handler)

            # A hardlink to the regular file
            info = tarfile.TarInfo(name="common/foo.txt")
            info.linkname = "common/subfolder/foo.txt"
            info.linkpath = "common/subfolder/foo.txt"
            info.name = "common/subfolder/bar/foo.txt"
            info.path = "common/subfolder/bar/foo.txt"
            info.type = b'1'  # This indicates a hardlink to the tgz file "common/subfolder/foo.txt"
            tgz.addfile(tarinfo=info, fileobj=None)
            tgz.close()

        assert not os.path.exists(os.path.join(tmp_folder, "subfolder", "foo.txt"))
        assert not os.path.exists(os.path.join(tmp_folder, "subfolder", "bar", "foo.txt"))
        untargz(tgz_path, destination=tmp_folder, strip_root=True)
        assert os.path.exists(os.path.join(tmp_folder, "subfolder", "foo.txt"))
        assert os.path.exists(os.path.join(tmp_folder, "subfolder", "bar", "foo.txt"))

        # Check develop2 public unzip
        rmdir(os.path.join(tmp_folder, "subfolder"))
        assert not os.path.exists(os.path.join(tmp_folder, "subfolder", "foo.txt"))
        assert not os.path.exists(os.path.join(tmp_folder, "subfolder", "bar", "foo.txt"))
        unzip_dev2(ConanFileMock(), tgz_path, destination=tmp_folder, strip_root=True)
        assert os.path.exists(os.path.join(tmp_folder, "subfolder", "foo.txt"))
        assert os.path.exists(os.path.join(tmp_folder, "subfolder", "bar", "foo.txt"))

    def test_plain_tgz(self):

        tmp_folder = temp_folder()
        with chdir(tmp_folder):
            # Create a couple of files
            ori_files_dir = os.path.join(tmp_folder, "subfolder-1.2.3")
            file1 = os.path.join(ori_files_dir, "file1")
            file2 = os.path.join(ori_files_dir, "folder", "file2")
            file3 = os.path.join(ori_files_dir, "file3")

            save(file1, "")
            save(file2, "")
            save(file3, "")

        tgz_folder = temp_folder()
        tgz_file = os.path.join(tgz_folder, "file.tar.gz")
        self._compress_folder(tmp_folder, tgz_file, folder_entry="subfolder-1.2.3")

        # Tgz unzipped regularly
        extract_folder = temp_folder()
        untargz(tgz_file, destination=extract_folder, strip_root=False)
        self.assertTrue(os.path.exists(os.path.join(extract_folder, "subfolder-1.2.3")))
        self.assertTrue(os.path.exists(os.path.join(extract_folder, "subfolder-1.2.3", "file1")))
        self.assertTrue(os.path.exists(os.path.join(extract_folder, "subfolder-1.2.3", "folder",
                                                    "file2")))
        self.assertTrue(os.path.exists(os.path.join(extract_folder, "subfolder-1.2.3", "file3")))

        # Extract without the subfolder
        extract_folder = temp_folder()
        untargz(tgz_file, destination=extract_folder, strip_root=True)
        self.assertFalse(os.path.exists(os.path.join(extract_folder, "subfolder-1.2.3")))
        self.assertTrue(os.path.exists(os.path.join(extract_folder, "file1")))
        self.assertTrue(os.path.exists(os.path.join(extract_folder, "folder", "file2")))
        self.assertTrue(os.path.exists(os.path.join(extract_folder, "file3")))

    def test_plain_tgz_common_base(self):

        tmp_folder = temp_folder()
        with chdir(tmp_folder):
            # Create a couple of files
            ori_files_dir = os.path.join(tmp_folder, "subfolder-1.2.3")
            file1 = os.path.join(ori_files_dir, "folder", "file1")
            file2 = os.path.join(ori_files_dir, "folder", "file2")
            file3 = os.path.join(ori_files_dir, "folder", "file3")

            save(file1, "")
            save(file2, "")
            save(file3, "")

        tgz_folder = temp_folder()
        tgz_file = os.path.join(tgz_folder, "file.tar.gz")
        self._compress_folder(tmp_folder, tgz_file)

        # Tgz unzipped regularly
        extract_folder = temp_folder()
        untargz(tgz_file, destination=extract_folder, strip_root=True)
        self.assertFalse(os.path.exists(os.path.join(extract_folder, "subfolder-1.2.3")))
        self.assertTrue(os.path.exists(os.path.join(extract_folder, "folder", "file1")))
        self.assertTrue(os.path.exists(os.path.join(extract_folder, "folder", "file2")))
        self.assertTrue(os.path.exists(os.path.join(extract_folder, "folder", "file3")))

    def test_invalid_flat(self):
        tmp_folder = temp_folder()
        with chdir(tmp_folder):
            # Not a single dir containing everything
            file1 = os.path.join(tmp_folder, "subfolder-1.2.3", "folder2", "file1")
            file2 = os.path.join(tmp_folder, "other-1.2.3", "folder", "file2")

            save(file1, "")
            save(file2, "")

        tgz_folder = temp_folder()
        tgz_file = os.path.join(tgz_folder, "file.tar.gz")
        self._compress_folder(tmp_folder, tgz_file)

        extract_folder = temp_folder()
        with six.assertRaisesRegex(self, ConanException, "The tgz file contains more than 1 folder "
                                                         "in the root"):
            untargz(tgz_file, destination=extract_folder, strip_root=True)

    def test_invalid_flat_single_file(self):
        tmp_folder = temp_folder()
        with chdir(tmp_folder):
            save("file1", "contentsfile1")

        zip_folder = temp_folder()
        tgz_file = os.path.join(zip_folder, "file.tar.gz")
        self._compress_folder(tmp_folder, tgz_file)

        # Extract without the subfolder
        extract_folder = temp_folder()
        with six.assertRaisesRegex(self, ConanException, "The tgz file contains a file in the root"):
            unzip(tgz_file, destination=extract_folder, strip_root=True, output=Mock())



import os
import textwrap
import unittest

from conans.model.ref import ConanFileReference
from conans.test.utils.test_files import temp_folder
from conans.test.utils.tools import TestClient
from conans.util.files import save_files, load


class ExportsMethodTest(unittest.TestCase):

    def test_export_method(self):
        client = TestClient()
        conanfile = textwrap.dedent("""
            from conans import ConanFile

            class MethodConan(ConanFile):
                exports = "file.txt"
                def export(self):
                    self.copy("LICENSE.md")
            """)
        client.save({"conanfile.py": conanfile, "LICENSE.md": "license", "file.txt": "file"})
        client.run("export . pkg/0.1@")
        self.assertIn("pkg/0.1: Calling export()", client.out)
        self.assertIn("pkg/0.1 exports: Copied 1 '.txt' file: file.txt", client.out)
        self.assertIn("pkg/0.1 export() method: Copied 1 '.md' file: LICENSE.md", client.out)

        layout = client.cache.package_layout(ConanFileReference.loads("pkg/0.1"))
        exported_files = os.listdir(layout.export())
        self.assertIn("file.txt", exported_files)
        self.assertIn("LICENSE.md", exported_files)

    def test_export_method_parent_folder(self):
        folder = temp_folder()
        save_files(folder, {"subdir/subdir2/file2.txt": "", "subdir/file1.txt": ""})
        client = TestClient(current_folder=os.path.join(folder, "recipe"))
        conanfile = textwrap.dedent("""
            from conans import ConanFile

            class MethodConan(ConanFile):
                def export(self):
                    self.output.info("Executing export() method")
                    self.copy("*.txt", src="../subdir")
            """)
        client.save({"conanfile.py": conanfile})
        client.run("export . pkg/0.1@")
        self.assertIn("pkg/0.1 export() method: Copied 2 '.txt' files: file1.txt, file2.txt",
                      client.out)

        layout = client.cache.package_layout(ConanFileReference.loads("pkg/0.1"))
        self.assertTrue(os.path.isfile(os.path.join(layout.export(), "file1.txt")))
        self.assertTrue(os.path.isfile(os.path.join(layout.export(), "subdir2", "file2.txt")))

    def test_export_no_settings_options_method(self):
        client = TestClient()
        conanfile = textwrap.dedent("""
            from conans import ConanFile

            class MethodConan(ConanFile):
                settings = "os"
                def export(self):
                    if self.settings.os == "Windows":
                        self.copy("LICENSE.md")
            """)
        client.save({"conanfile.py": conanfile, "LICENSE.md": "license"})
        client.run("export . pkg/0.1@", assert_error=True)
        self.assertIn("ERROR: pkg/0.1: Error in export() method, line 7", client.out)

        conanfile = textwrap.dedent("""
            from conans import ConanFile

            class MethodConan(ConanFile):
                default_options = {"myopt": "myval"}
                def export(self):
                    if self.default_options["myopt"] == "myval":
                        self.copy("LICENSE.md")
            """)
        client.save({"conanfile.py": conanfile})
        client.run("export . pkg/0.1@", assert_error=True)
        self.assertIn("ERROR: pkg/0.1: Error in export() method, line 7", client.out)

        conanfile = textwrap.dedent("""
            from conans import ConanFile

            class MethodConan(ConanFile):
                options = {"myopt": ["myval"]}
                default_options = {"myopt": "myval"}
                def export(self):
                    pass
                def export_sources(self):
                    pass
                def build(self):
                    self.output.info("MYOPT: %s" % self.options.myopt)
            """)
        client.save({"conanfile.py": conanfile})
        client.run("create . pkg/0.1@")
        self.assertIn("pkg/0.1: MYOPT: myval", client.out)

    def test_export_folders(self):
        client = TestClient()
        conanfile = textwrap.dedent("""
           from conans import ConanFile
           from conans.tools import save, load
           import os

           class MethodConan(ConanFile):
               def export(self):
                   content = load(os.path.join(os.getcwd(), "data.txt"))
                   save(os.path.join(self.export_folder, "myfile.txt"), content)
           """)
        client.save({"recipe/conanfile.py": conanfile, "recipe/data.txt": "mycontent"})
        client.run("export recipe pkg/0.1@")
        layout = client.cache.package_layout(ConanFileReference.loads("pkg/0.1"))
        self.assertEqual("mycontent", load(os.path.join(layout.export(), "myfile.txt")))
        client.current_folder = os.path.join(client.current_folder, "recipe")
        client.run("export . pkg/0.1@")
        layout = client.cache.package_layout(ConanFileReference.loads("pkg/0.1"))
        self.assertEqual("mycontent", load(os.path.join(layout.export(), "myfile.txt")))

    def test_export_attribute_error(self):
        client = TestClient()
        conanfile = textwrap.dedent("""
            from conans import ConanFile
            class MethodConan(ConanFile):
                export = "file.txt"
            """)
        client.save({"conanfile.py": conanfile, "file.txt": "file"})
        client.run("export . pkg/0.1@", assert_error=True)
        self.assertIn("ERROR: conanfile 'export' must be a method", client.out)

    def test_exports_method_error(self):
        client = TestClient()
        conanfile = textwrap.dedent("""
            from conans import ConanFile
            class MethodConan(ConanFile):
                def exports(self):
                    pass
            """)
        client.save({"conanfile.py": conanfile})
        client.run("export . pkg/0.1@", assert_error=True)
        self.assertIn("ERROR: conanfile 'exports' shouldn't be a method, use 'export()' instead",
                      client.out)


class ExportsSourcesMethodTest(unittest.TestCase):

    def test_export_sources_method(self):
        client = TestClient()
        conanfile = textwrap.dedent("""
            from conans import ConanFile

            class MethodConan(ConanFile):
                exports_sources = "file.txt"
                def export_sources(self):
                    self.copy("LICENSE.md")
            """)
        client.save({"conanfile.py": conanfile, "LICENSE.md": "license", "file.txt": "file"})
        client.run("export . pkg/0.1@")
        self.assertIn("pkg/0.1 exports_sources: Copied 1 '.txt' file: file.txt", client.out)
        self.assertIn("pkg/0.1 export_sources() method: Copied 1 '.md' file: LICENSE.md", client.out)

        layout = client.cache.package_layout(ConanFileReference.loads("pkg/0.1"))
        exported_files = os.listdir(layout.export_sources())
        self.assertIn("file.txt", exported_files)
        self.assertIn("LICENSE.md", exported_files)

    def test_export_source_folders(self):
        client = TestClient()
        conanfile = textwrap.dedent("""
           from conans import ConanFile
           from conans.tools import save, load
           import os

           class MethodConan(ConanFile):
               def export_sources(self):
                   content = load(os.path.join(os.getcwd(), "data.txt"))
                   save(os.path.join(self.export_sources_folder, "myfile.txt"), content)
           """)
        client.save({"recipe/conanfile.py": conanfile, "recipe/data.txt": "mycontent"})
        client.run("export recipe pkg/0.1@")
        layout = client.cache.package_layout(ConanFileReference.loads("pkg/0.1"))
        self.assertEqual("mycontent", load(os.path.join(layout.export_sources(), "myfile.txt")))
        client.current_folder = os.path.join(client.current_folder, "recipe")
        client.run("export . pkg/0.1@")
        layout = client.cache.package_layout(ConanFileReference.loads("pkg/0.1"))
        self.assertEqual("mycontent", load(os.path.join(layout.export_sources(), "myfile.txt")))

    def test_export_sources_attribute_error(self):
        client = TestClient()
        conanfile = textwrap.dedent("""
            from conans import ConanFile
            class MethodConan(ConanFile):
                export_sources = "file.txt"
            """)
        client.save({"conanfile.py": conanfile, "file.txt": "file"})
        client.run("export . pkg/0.1@", assert_error=True)
        self.assertIn("ERROR: conanfile 'export_sources' must be a method", client.out)

    def test_exports_sources_method_error(self):
        client = TestClient()
        conanfile = textwrap.dedent("""
            from conans import ConanFile
            class MethodConan(ConanFile):
                def exports_sources(self):
                    pass
            """)
        client.save({"conanfile.py": conanfile})
        client.run("export . pkg/0.1@", assert_error=True)
        self.assertIn("ERROR: conanfile 'exports_sources' shouldn't be a method, "
                      "use 'export_sources()' instead", client.out)

    def test_exports_sources_upload_error(self):
        # https://github.com/conan-io/conan/issues/7377
        client = TestClient(default_server_user=True)
        conanfile = textwrap.dedent("""
            from conans import ConanFile, tools

            class MethodConan(ConanFile):
                def export_sources(self):
                    self.copy("*")
                def build(self):
                    self.output.info("CONTENT: %s" % tools.load("myfile.txt"))
            """)
        client.save({"conanfile.py": conanfile,
                     "myfile.txt": "mycontent"})
        client.run("export . pkg/0.1@")
        self.assertIn("pkg/0.1 export_sources() method: Copied 1 '.txt' file: myfile.txt",
                      client.out)
        client.run("upload pkg/0.1@")
        client.run("remove * -f")
        client.run("install pkg/0.1@ --build")
        self.assertIn("Downloading conan_sources.tgz", client.out)
        self.assertIn("pkg/0.1: CONTENT: mycontent", client.out)



import sys
import textwrap

import pytest

from conan.tools.meson import MesonToolchain
from conans.test.utils.tools import TestClient


@pytest.mark.skipif(sys.version_info.major == 2, reason="Meson not supported in Py2")
def test_apple_meson_keep_user_custom_flags():
    default = textwrap.dedent("""
    [settings]
    os=Macos
    arch=x86_64
    compiler=apple-clang
    compiler.version=12.0
    compiler.libcxx=libc++
    build_type=Release
    """)

    cross = textwrap.dedent("""
    [settings]
    os = iOS
    os.version = 10.0
    os.sdk = iphoneos
    arch = armv8
    compiler = apple-clang
    compiler.version = 12.0
    compiler.libcxx = libc++

    [conf]
    tools.apple:sdk_path=/my/sdk/path
    """)

    _conanfile_py = textwrap.dedent("""
    from conan import ConanFile
    from conan.tools.meson import MesonToolchain

    class App(ConanFile):
        settings = "os", "arch", "compiler", "build_type"

        def generate(self):
            tc = MesonToolchain(self)
            # Customized apple flags
            tc.apple_arch_flag = ['-arch', 'myarch']
            tc.apple_isysroot_flag = ['-isysroot', '/other/sdk/path']
            tc.apple_min_version_flag = ['-otherminversion=10.7']
            tc.generate()
    """)

    t = TestClient()
    t.save({"conanfile.py": _conanfile_py,
            "build_prof": default,
            "host_prof": cross})

    t.run("install . -pr:h host_prof -pr:b build_prof")
    content = t.load(MesonToolchain.cross_filename)
    assert "c_args = ['-isysroot', '/other/sdk/path', '-arch', 'myarch', '-otherminversion=10.7']" in content
    assert "c_link_args = ['-isysroot', '/other/sdk/path', '-arch', 'myarch', '-otherminversion=10.7']" in content
    assert "cpp_args = ['-isysroot', '/other/sdk/path', '-arch', 'myarch', '-otherminversion=10.7']" in content
    assert "cpp_link_args = ['-isysroot', '/other/sdk/path', '-arch', 'myarch', '-otherminversion=10.7']" in content


@pytest.mark.skipif(sys.version_info.major == 2, reason="Meson not supported in Py2")
def test_extra_flags_via_conf():
    profile = textwrap.dedent("""
        [settings]
        os=Windows
        arch=x86_64
        compiler=gcc
        compiler.version=9
        compiler.cppstd=17
        compiler.libcxx=libstdc++11
        build_type=Release

        [buildenv]
        CFLAGS=-flag0 -other=val
        CXXFLAGS=-flag0 -other=val
        LDFLAGS=-flag0 -other=val

        [conf]
        tools.build:cxxflags=["-flag1", "-flag2"]
        tools.build:cflags=["-flag3", "-flag4"]
        tools.build:sharedlinkflags+=["-flag5"]
        tools.build:exelinkflags+=["-flag6"]
   """)
    t = TestClient()
    t.save({"conanfile.txt": "[generators]\nMesonToolchain",
            "profile": profile})

    t.run("install . -pr=profile")
    content = t.load(MesonToolchain.native_filename)
    assert "cpp_args = ['-flag0', '-other=val', '-flag1', '-flag2']" in content
    assert "c_args = ['-flag0', '-other=val', '-flag3', '-flag4']" in content
    assert "c_link_args = ['-flag0', '-other=val', '-flag5', '-flag6']" in content
    assert "cpp_link_args = ['-flag0', '-other=val', '-flag5', '-flag6']" in content


@pytest.mark.skipif(sys.version_info.major == 2, reason="Meson not supported in Py2")
def test_correct_quotes():
    profile = textwrap.dedent("""
       [settings]
       os=Windows
       arch=x86_64
       compiler=gcc
       compiler.version=9
       compiler.cppstd=17
       compiler.libcxx=libstdc++11
       build_type=Release
       """)
    t = TestClient()
    t.save({"conanfile.txt": "[generators]\nMesonToolchain",
            "profile": profile})

    t.run("install . -pr=profile")
    content = t.load(MesonToolchain.native_filename)
    assert "cpp_std = 'c++17'" in content
    assert "backend = 'ninja'" in content
    assert "buildtype = 'release'" in content



import unittest
from collections import OrderedDict

from conans.model.ref import ConanFileReference

from conans.test.utils.tools import TestServer, TurboTestClient, GenConanfile


class InstallCascadeTest(unittest.TestCase):

    def setUp(self):
        """
         A
        / \
       B   C
       |    \
       D    |
      / \   |
      |  \ /
      E   F
        """
        server = TestServer()
        servers = OrderedDict([("default", server)])
        self.client = TurboTestClient(servers=servers)

        self.ref_a = ConanFileReference.loads("libA/1.0@conan/stable")
        self.client.create(self.ref_a, conanfile=GenConanfile())

        self.ref_b = ConanFileReference.loads("libB/1.0@conan/stable")
        self.client.create(self.ref_b, conanfile=GenConanfile().with_requirement(self.ref_a))

        self.ref_c = ConanFileReference.loads("libC/1.0@conan/stable")
        self.client.create(self.ref_c, conanfile=GenConanfile().with_requirement(self.ref_a))

        self.ref_d = ConanFileReference.loads("libD/1.0@conan/stable")
        self.client.create(self.ref_d, conanfile=GenConanfile().with_requirement(self.ref_b))

        self.ref_e = ConanFileReference.loads("libE/1.0@conan/stable")
        self.client.create(self.ref_e, conanfile=GenConanfile().with_requirement(self.ref_d))

        self.ref_f = ConanFileReference.loads("libF/1.0@conan/stable")
        conanfile = GenConanfile().with_requirement(self.ref_c).with_requirement(self.ref_d)
        self.client.create(self.ref_f, conanfile=conanfile)

    def _assert_built(self, refs):
        for ref in refs:
            self.assertIn("{}: Copying sources to build folder".format(ref), self.client.out)
        for ref in [self.ref_a, self.ref_b, self.ref_c, self.ref_d, self.ref_e, self.ref_f]:
            if ref not in refs:
                self.assertNotIn("{}: Copying sources to build folder".format(ref),
                                 self.client.out)

    def test_install_cascade_only_affected(self):
        project = ConanFileReference.loads("project/1.0@conan/stable")
        project_cf = GenConanfile().with_requirement(self.ref_e).with_requirement(self.ref_f)

        # Building A everything is built
        self.client.create(project, conanfile=project_cf,
                           args="--build {} --build cascade".format(self.ref_a))
        self._assert_built([self.ref_a, self.ref_b, self.ref_c, self.ref_d,
                            self.ref_e, self.ref_f, project])

        # Building D builds E, F and project
        self.client.create(project, conanfile=project_cf,
                           args="--build {} --build cascade".format(self.ref_d))
        self._assert_built([self.ref_d, self.ref_e, self.ref_f, project])

        # Building E only builds E and project
        self.client.create(project, conanfile=project_cf,
                           args="--build {} --build cascade".format(self.ref_e))
        self._assert_built([self.ref_e, project])

        # Building project only builds project
        self.client.create(project, conanfile=project_cf,
                           args="--build {} --build cascade".format(project))
        self._assert_built([project])

        # Building C => builds F and project
        self.client.create(project, conanfile=project_cf,
                           args="--build {} --build cascade".format(self.ref_c))
        self._assert_built([project, self.ref_f, self.ref_c])



# coding=utf-8

import unittest

from conans.test.utils.tools import TestClient, GenConanfile


class VersionRangeNoUserChannelTestCase(unittest.TestCase):

    def test_match(self):
        t = TestClient()
        t.save({"conanfile.py": GenConanfile(), })
        t.run("export . libB/1.0@")

        # Use version ranges without user/channel
        t.save({"conanfile.py": GenConanfile().with_require("libB/[~1.x]")})
        t.run("info . --only requires")
        self.assertIn("Version range '~1.x' required by 'conanfile.py'"
                      " resolved to 'libB/1.0' in local cache", t.out)

    def test_no_match(self):
        t = TestClient()
        t.save({"conanfile.py": GenConanfile()})
        t.run("export . libB/1.0@user/channel")

        # Use version ranges without user/channel
        t.save({"conanfile.py": GenConanfile().with_require("libB/[~1.x]")})
        t.run("info . --only requires", assert_error=True)
        self.assertIn("ERROR: Version range '~1.x' from requirement 'libB/[~1.x]' required"
                      " by 'conanfile.py' could not be resolved in local cache", t.out)

    def test_no_match_with_userchannel(self):
        t = TestClient()
        t.save({"conanfile.py": GenConanfile()})
        t.run("export . libB/1.0@")

        # Use version ranges with user/channel
        t.save({"conanfile.py": GenConanfile().with_require("libB/[~1.x]@user/channel")})
        t.run("info . --only requires", assert_error=True)
        self.assertIn("ERROR: Version range '~1.x' from requirement 'libB/[~1.x]@user/channel'"
                      " required by 'conanfile.py' could not be resolved in local cache", t.out)

    def test_mixed_user_channel(self):
        # https://github.com/conan-io/conan/issues/7846
        t = TestClient(default_server_user=True)
        t.save({"conanfile.py": GenConanfile()})
        t.run("create . pkg/1.0@")
        t.run("create . pkg/1.1@")
        t.run("create . pkg/2.0@")
        t.run("create . pkg/1.0@user/testing")
        t.run("create . pkg/1.1@user/testing")
        t.run("create . pkg/2.0@user/testing")
        t.run("upload * --all --confirm")
        t.run("remove * -f")

        t.run('install "pkg/[>0 <2]@"')
        self.assertIn("pkg/1.1 from 'default' - Downloaded", t.out)
        t.run('install "pkg/[>0 <2]@user/testing"')
        self.assertIn("pkg/1.1@user/testing from 'default' - Downloaded", t.out)



import os
import textwrap
import unittest

from parameterized.parameterized import parameterized

from conans.model.ref import ConanFileReference
from conans.paths import CONANFILE
from conans.test.utils.tools import TestClient, GenConanfile

tool_conanfile = """from conans import ConanFile

class Tool(ConanFile):
    name = "Tool"
    version = "0.1"

    def package_info(self):
        self.env_info.TOOL_PATH.append("MyToolPath")
"""

tool_conanfile2 = tool_conanfile.replace("0.1", "0.3")

conanfile = """
import os
from conans import ConanFile, tools

class MyLib(ConanFile):
    name = "MyLib"
    version = "0.1"
    {}

    def build(self):
        self.output.info("ToolPath: %s" % os.getenv("TOOL_PATH"))
"""

requires = conanfile.format('build_requires = "Tool/0.1@lasote/stable"')
requires_range = conanfile.format('build_requires = "Tool/[>0.0]@lasote/stable"')
requirements = conanfile.format("""def build_requirements(self):
        self.build_requires("Tool/0.1@lasote/stable")""")
override = conanfile.format("""build_requires = "Tool/0.2@user/channel"

    def build_requirements(self):
        self.build_requires("Tool/0.1@lasote/stable")""")


profile = """
[build_requires]
Tool/0.3@lasote/stable
nonexistingpattern*: SomeTool/1.2@user/channel
"""


class BuildRequiresTest(unittest.TestCase):

    def test_consumer(self):
        # https://github.com/conan-io/conan/issues/5425
        catch_ref = ConanFileReference.loads("catch/0.1@user/testing")
        libA_ref = ConanFileReference.loads("LibA/0.1@user/testing")

        t = TestClient()
        t.save({"conanfile.py":
                    GenConanfile().with_package_info(cpp_info={"libs": ["mylibcatch0.1lib"]},
                                                     env_info={"MYENV": ["myenvcatch0.1env"]})})
        t.run("create . catch/0.1@user/testing")
        t.save({"conanfile.py": GenConanfile().with_require(catch_ref, private=True)})
        t.run("create . LibA/0.1@user/testing")
        t.save({"conanfile.py": GenConanfile().with_require(libA_ref)
                                              .with_build_requires(catch_ref)})
        t.run("install .")
        self.assertIn("catch/0.1@user/testing from local cache", t.out)
        self.assertIn("catch/0.1@user/testing:5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9 - Skip",
                      t.out)
        self.assertIn("catch/0.1@user/testing:5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9 - Cache",
                      t.out)
        conanbuildinfo = t.load("conanbuildinfo.txt")
        self.assertIn('MYENV=["myenvcatch0.1env"]', conanbuildinfo)
        self.assertIn('[libs_catch]', conanbuildinfo)
        self.assertIn("mylibcatch0.1lib", conanbuildinfo)

    def test_build_requires_diamond(self):
        libA_ref = ConanFileReference.loads("libA/0.1@user/testing")
        libB_ref = ConanFileReference.loads("libB/0.1@user/testing")

        t = TestClient()
        t.save({"conanfile.py": GenConanfile()})
        t.run("create . libA/0.1@user/testing")

        t.save({"conanfile.py": GenConanfile().with_require(libA_ref)})
        t.run("create . libB/0.1@user/testing")

        t.save({"conanfile.py": GenConanfile().with_build_requires(libB_ref)
                                              .with_build_requires(libA_ref)})
        t.run("create . libC/0.1@user/testing")
        self.assertIn("libC/0.1@user/testing: Created package", t.out)

    def test_create_with_tests_and_build_requires(self):
        client = TestClient()
        # Generate and export the build_require recipe
        conanfile = """from conans import ConanFile
class MyBuildRequire(ConanFile):
    def package_info(self):
        self.env_info.MYVAR="1"
"""
        client.save({"conanfile.py": conanfile})
        client.run("create . Build1/0.1@conan/stable")
        client.save({"conanfile.py": conanfile.replace('MYVAR="1"', 'MYVAR2="2"')})
        client.run("create . Build2/0.1@conan/stable")

        # Create a recipe that will use a profile requiring the build_require
        client.save({"conanfile.py": """from conans import ConanFile
import os

class MyLib(ConanFile):
    build_requires = "Build2/0.1@conan/stable"
    def build(self):
        assert(os.environ['MYVAR']=='1')
        assert(os.environ['MYVAR2']=='2')

""", "myprofile": '''
[build_requires]
Build1/0.1@conan/stable
''',
                    "test_package/conanfile.py": """from conans import ConanFile
import os

class MyTest(ConanFile):
    def build(self):
        assert(os.environ['MYVAR']=='1')
    def test(self):
        self.output.info("TESTING!!!")
"""}, clean_first=True)

        # Test that the build require is applyed to testing
        client.run("create . Lib/0.1@conan/stable --profile=./myprofile")
        self.assertEqual(1, str(client.out).count("Lib/0.1@conan/stable: "
                                                  "Applying build-requirement:"
                                                  " Build1/0.1@conan/stable"))
        self.assertIn("TESTING!!", client.out)

    def test_dependents_txt(self):
        client = TestClient()
        boost = """from conans import ConanFile
class Boost(ConanFile):
    def package_info(self):
        self.env_info.PATH.append("myboostpath")
"""
        client.save({CONANFILE: boost})
        client.run("create . Boost/1.0@user/channel")
        other = """[build_requires]
Boost/1.0@user/channel
"""
        client.save({"conanfile.txt": other}, clean_first=True)
        client.run("install .")

        self.assertIn("""Build requirements
    Boost/1.0@user/channel""", client.out)
        conanbuildinfo = client.load("conanbuildinfo.txt")
        self.assertIn('PATH=["myboostpath"]', conanbuildinfo)

    def test_dependents(self):
        client = TestClient()
        boost = """from conans import ConanFile
class Boost(ConanFile):
    def package_info(self):
        self.env_info.PATH.append("myboostpath")
"""
        client.save({CONANFILE: boost})
        client.run("create . Boost/1.0@user/channel")
        other = """from conans import ConanFile
import os
class Other(ConanFile):
    requires = "Boost/1.0@user/channel"
    def build(self):
        self.output.info("OTHER PATH FOR BUILD %s" % os.getenv("PATH"))
    def package_info(self):
        self.env_info.PATH.append("myotherpath")
"""
        client.save({CONANFILE: other})
        client.run("create . Other/1.0@user/channel")
        lib = """from conans import ConanFile
import os
class Lib(ConanFile):
    build_requires = "Boost/1.0@user/channel", "Other/1.0@user/channel"
    def build(self):
        self.output.info("LIB PATH FOR BUILD %s" % os.getenv("PATH"))
"""
        client.save({CONANFILE: lib})
        client.run("create . Lib/1.0@user/channel")
        self.assertIn("LIB PATH FOR BUILD myotherpath%smyboostpath" % os.pathsep,
                      client.out)

    def test_applyname(self):
        # https://github.com/conan-io/conan/issues/4135
        client = TestClient()
        mingw = """from conans import ConanFile
class Tool(ConanFile):
    def package_info(self):
        self.env_info.PATH.append("mymingwpath")
"""
        myprofile = """
[build_requires]
consumer*: mingw/0.1@myuser/stable
"""
        app = """from conans import ConanFile
import os
class App(ConanFile):
    name = "consumer"
    def build(self):
        self.output.info("APP PATH FOR BUILD %s" % os.getenv("PATH"))
"""
        client.save({CONANFILE: mingw})
        client.run("create . mingw/0.1@myuser/stable")
        client.save({CONANFILE: app,
                     "myprofile": myprofile})
        client.run("install . -pr=myprofile")
        self.assertIn("conanfile.py (consumer/None): Applying build-requirement: "
                      "mingw/0.1@myuser/stable", client.out)
        client.run("build .")
        self.assertIn("conanfile.py (consumer/None): APP PATH FOR BUILD mymingwpath",
                      client.out)

    def test_transitive(self):
        client = TestClient()
        mingw = """from conans import ConanFile
class Tool(ConanFile):
    def package_info(self):
        self.env_info.PATH.append("mymingwpath")
"""
        myprofile = """
[build_requires]
mingw/0.1@lasote/stable
"""
        gtest = """from conans import ConanFile
import os
class Gtest(ConanFile):
    def build(self):
        self.output.info("GTEST PATH FOR BUILD %s" % os.getenv("PATH"))
"""
        app = """from conans import ConanFile
import os
class App(ConanFile):
    build_requires = "gtest/0.1@lasote/stable"
    def build(self):
        self.output.info("APP PATH FOR BUILD %s" % os.getenv("PATH"))
"""
        client.save({CONANFILE: mingw})
        client.run("create . mingw/0.1@lasote/stable")
        client.save({CONANFILE: gtest})
        client.run("export . gtest/0.1@lasote/stable")
        client.save({CONANFILE: app,
                     "myprofile": myprofile})
        client.run("create . app/0.1@lasote/stable --build=missing -pr=myprofile")
        self.assertIn("app/0.1@lasote/stable: APP PATH FOR BUILD mymingwpath",
                      client.out)
        self.assertIn("gtest/0.1@lasote/stable: GTEST PATH FOR BUILD mymingwpath",
                      client.out)

    def test_profile_order(self):
        client = TestClient()
        mingw = """from conans import ConanFile
class Tool(ConanFile):
    def package_info(self):
        self.env_info.PATH.append("mymingwpath")
"""
        msys = """from conans import ConanFile
class Tool(ConanFile):
    def package_info(self):
        self.env_info.PATH.append("mymsyspath")
"""
        myprofile1 = """
[build_requires]
mingw/0.1@lasote/stable
msys/0.1@lasote/stable
"""
        myprofile2 = """
[build_requires]
msys/0.1@lasote/stable
mingw/0.1@lasote/stable
"""

        app = """from conans import ConanFile
import os
class App(ConanFile):
    def build(self):
        self.output.info("APP PATH FOR BUILD %s" % os.getenv("PATH"))
"""
        client.save({CONANFILE: mingw})
        client.run("create . mingw/0.1@lasote/stable")
        client.save({CONANFILE: msys})
        client.run("create . msys/0.1@lasote/stable")
        client.save({CONANFILE: app,
                     "myprofile1": myprofile1,
                     "myprofile2": myprofile2})
        client.run("create . app/0.1@lasote/stable -pr=myprofile1")
        self.assertIn("app/0.1@lasote/stable: APP PATH FOR BUILD mymingwpath%smymsyspath"
                      % os.pathsep, client.out)
        client.run("create . app/0.1@lasote/stable -pr=myprofile2")
        self.assertIn("app/0.1@lasote/stable: APP PATH FOR BUILD mymsyspath%smymingwpath"
                      % os.pathsep, client.out)

    def test_require_itself(self):
        client = TestClient()
        mytool_conanfile = """from conans import ConanFile
class Tool(ConanFile):
    def build(self):
        self.output.info("BUILDING MYTOOL")
"""
        myprofile = """
[build_requires]
Tool/0.1@lasote/stable
"""
        client.save({CONANFILE: mytool_conanfile,
                     "profile.txt": myprofile})
        client.run("create . Tool/0.1@lasote/stable -pr=profile.txt")
        self.assertEqual(1, str(client.out).count("BUILDING MYTOOL"))

    @parameterized.expand([(requires, ), (requires_range, ), (requirements, ), (override, )])
    def test_build_requires(self, conanfile):
        client = TestClient()
        client.save({CONANFILE: tool_conanfile})
        client.run("export . lasote/stable")

        client.save({CONANFILE: conanfile}, clean_first=True)
        client.run("export . lasote/stable")

        client.run("install MyLib/0.1@lasote/stable --build missing")
        self.assertIn("Tool/0.1@lasote/stable: Generating the package", client.out)
        self.assertIn("ToolPath: MyToolPath", client.out)

        client.run("install MyLib/0.1@lasote/stable")
        self.assertNotIn("Tool", client.out)
        self.assertIn("MyLib/0.1@lasote/stable: Already installed!", client.out)

    @parameterized.expand([(requires, ), (requires_range, ), (requirements, ), (override, )])
    def test_profile_override(self, conanfile):
        client = TestClient()
        client.save({CONANFILE: tool_conanfile2}, clean_first=True)
        client.run("export . lasote/stable")

        client.save({CONANFILE: conanfile,
                     "profile.txt": profile,
                     "profile2.txt": profile.replace("0.3", "[>0.2]")}, clean_first=True)
        client.run("export . lasote/stable")

        client.run("install MyLib/0.1@lasote/stable --profile ./profile.txt --build missing")
        self.assertNotIn("Tool/0.1", client.out)
        self.assertNotIn("Tool/0.2", client.out)
        self.assertIn("Tool/0.3@lasote/stable: Generating the package", client.out)
        self.assertIn("ToolPath: MyToolPath", client.out)

        client.run("install MyLib/0.1@lasote/stable")
        self.assertNotIn("Tool", client.out)
        self.assertIn("MyLib/0.1@lasote/stable: Already installed!", client.out)

        client.run("install MyLib/0.1@lasote/stable --profile ./profile2.txt --build")
        self.assertNotIn("Tool/0.1", client.out)
        self.assertNotIn("Tool/0.2", client.out)
        self.assertIn("Tool/0.3@lasote/stable: Generating the package", client.out)
        self.assertIn("ToolPath: MyToolPath", client.out)

    def test_options(self):
        conanfile = """from conans import ConanFile
class package(ConanFile):
    name            = "first"
    version         = "0.0.0"
    options         = {"coverage": [True, False]}
    default_options = "coverage=False"
    def build(self):
        self.output.info("Coverage: %s" % self.options.coverage)
    """
        client = TestClient()
        client.save({"conanfile.py": conanfile})
        client.run("export . lasote/stable")

        consumer = """from conans import ConanFile

class package(ConanFile):
    name            = "second"
    version         = "0.0.0"
    default_options = "first:coverage=True"
    build_requires  = "first/0.0.0@lasote/stable"
"""
        client.save({"conanfile.py": consumer})
        client.run("install . --build=missing -o Pkg:someoption=3")
        self.assertIn("first/0.0.0@lasote/stable: Coverage: True", client.out)

    def test_failed_assert(self):
        # https://github.com/conan-io/conan/issues/5685
        client = TestClient()
        client.save({"conanfile.py": GenConanfile()})
        client.run("export . common/1.0@test/test")

        req = textwrap.dedent("""
            from conans import ConanFile
            class BuildReqConan(ConanFile):
                requires = "common/1.0@test/test"
            """)
        client.save({"conanfile.py": req})
        client.run("export . req/1.0@test/test")
        client.run("export . build_req/1.0@test/test")

        build_req_req = textwrap.dedent("""
            from conans import ConanFile
            class BuildReqConan(ConanFile):
                requires = "common/1.0@test/test"
                build_requires = "build_req/1.0@test/test"
        """)
        client.save({"conanfile.py": build_req_req})
        client.run("export . build_req_req/1.0@test/test")

        consumer = textwrap.dedent("""
                    [requires]
                    req/1.0@test/test
                    [build_requires]
                    build_req_req/1.0@test/test
                """)
        client.save({"conanfile.txt": consumer}, clean_first=True)
        client.run("install . --build=missing")
        # This used to assert and trace, now it works
        self.assertIn("conanfile.txt: Applying build-requirement: build_req_req/1.0@test/test",
                      client.out)

    def test_missing_transitive_dependency(self):
        # https://github.com/conan-io/conan/issues/5682
        client = TestClient()
        zlib = textwrap.dedent("""
            from conans import ConanFile
            class ZlibPkg(ConanFile):
                def package_info(self):
                    self.cpp_info.libs = ["myzlib"]
            """)
        client.save({"conanfile.py": zlib})
        client.run("export . zlib/1.0@test/test")

        client.save({"conanfile.py": GenConanfile().with_require("zlib/1.0@test/test")})
        client.run("export . freetype/1.0@test/test")
        client.save({"conanfile.py": GenConanfile().with_require("freetype/1.0@test/test")})
        client.run("export . fontconfig/1.0@test/test")
        harfbuzz = textwrap.dedent("""
            from conans import ConanFile
            class harfbuzz(ConanFile):
                requires = "freetype/1.0@test/test", "fontconfig/1.0@test/test"
                def build(self):
                     self.output.info("ZLIBS LIBS: %s" %self.deps_cpp_info["zlib"].libs)
            """)
        client.save({"conanfile.py": harfbuzz})
        client.run("export . harfbuzz/1.0@test/test")

        client.save({"conanfile.py": GenConanfile()
                    .with_build_requires("fontconfig/1.0@test/test")
                    .with_build_requires("harfbuzz/1.0@test/test")})
        client.run("install . --build=missing")
        self.assertIn("ZLIBS LIBS: ['myzlib']", client.out)


def test_dependents_new_buildenv():
    client = TestClient()
    boost = textwrap.dedent("""
        from conans import ConanFile
        class Boost(ConanFile):
            def package_info(self):
                self.buildenv_info.define_path("PATH", "myboostpath")
        """)
    other = textwrap.dedent("""
        from conans import ConanFile
        class Other(ConanFile):
            requires = "boost/1.0"
            def package_info(self):
                self.buildenv_info.append_path("PATH", "myotherpath")
                self.buildenv_info.prepend_path("PATH", "myotherprepend")
        """)
    consumer = textwrap.dedent("""
       from conans import ConanFile
       from conan.tools.env import VirtualBuildEnv
       import os
       class Lib(ConanFile):
           build_requires = {}
           def generate(self):
               build_env = VirtualBuildEnv(self).vars()
               with build_env.apply():
                   self.output.info("LIB PATH %s" % os.getenv("PATH"))
       """)
    client.save({"boost/conanfile.py": boost,
                 "other/conanfile.py": other,
                 "consumer/conanfile.py": consumer.format('"boost/1.0", "other/1.0"'),
                 "profile_define": "[buildenv]\nPATH=(path)profilepath",
                 "profile_append": "[buildenv]\nPATH+=(path)profilepath",
                 "profile_prepend": "[buildenv]\nPATH=+(path)profilepath"})
    client.run("create boost boost/1.0@")
    client.run("create other other/1.0@")
    client.run("install consumer")
    result = os.pathsep.join(["myotherprepend", "myboostpath", "myotherpath"])
    assert "LIB PATH {}".format(result) in client.out

    # Now test if we declare in different order, still topological order should be respected
    client.save({"consumer/conanfile.py": consumer.format('"other/1.0", "boost/1.0"')})
    client.run("install consumer")
    assert "LIB PATH {}".format(result) in client.out

    client.run("install consumer -pr=profile_define")
    assert "LIB PATH profilepath" in client.out
    client.run("install consumer -pr=profile_append")
    result = os.pathsep.join(["myotherprepend", "myboostpath", "myotherpath", "profilepath"])
    assert "LIB PATH {}".format(result) in client.out
    client.run("install consumer -pr=profile_prepend")
    result = os.pathsep.join(["profilepath", "myotherprepend", "myboostpath", "myotherpath"])
    assert "LIB PATH {}".format(result) in client.out


def test_tool_requires_conanfile_txt():
    client = TestClient()
    client.save({"conanfile.py": GenConanfile()})

    build_req = textwrap.dedent("""
        from conans import ConanFile
        class BuildReqConan(ConanFile):
            pass
        """)

    client.save({"conanfile.py": build_req})
    client.run("export . build_req/1.0@test/test")

    consumer = textwrap.dedent("""
                [tool_requires]
                build_req/1.0@test/test
            """)
    client.save({"conanfile.txt": consumer}, clean_first=True)
    client.run("install . --build=missing")
    assert "Applying build-requirement: build_req/1.0@test/test" in client.out



import os
import platform
import textwrap
import unittest

import pytest
from jinja2 import Template
from parameterized import parameterized

from conans.model.ref import ConanFileReference
from conans.test.utils.tools import TestClient


@pytest.mark.tool_cmake(version="3.19")
class LinkOrderTest(unittest.TestCase):
    """ Check that the link order of libraries is preserved when using CMake generators
        https://github.com/conan-io/conan/issues/6280
    """

    conanfile = Template(textwrap.dedent("""
        from conans import ConanFile

        class Recipe(ConanFile):
            name = "{{ref.name}}"
            version = "{{ref.version}}"

            {% if requires %}
            requires =
                {%- for req in requires -%}
                "{{ req }}"{% if not loop.last %}, {% endif %}
                {%- endfor -%}
            {% endif %}

            def build(self):
                with open("lib" + self.name + ".a", "w+") as f:
                    f.write("fake library content")
                with open(self.name + ".lib", "w+") as f:
                    f.write("fake library content")

                {% for it in libs_extra %}
                with open("lib{{ it }}.a", "w+") as f:
                    f.write("fake library content")
                with open("{{ it }}.lib", "w+") as f:
                    f.write("fake library content")
                {% endfor %}

            def package(self):
                self.copy("*.a", dst="lib")
                self.copy("*.lib", dst="lib")

            def package_info(self):
                # Libraries
                self.cpp_info.libs = ["{{ ref.name }}"]
                {% for it in libs_extra %}
                self.cpp_info.libs.append("{{ it }}")
                {% endfor %}
                {% for it in libs_system %}
                self.cpp_info.libs.append("{{ it }}")
                {% endfor %}

                {% if system_libs %}
                self.cpp_info.system_libs = [
                    {%- for it in system_libs -%}
                    "{{ it }}"{% if not loop.last %}, {% endif %}
                    {%- endfor -%}]
                {% endif %}

                {% if frameworks %}
                self.cpp_info.frameworks.extend([
                    {%- for it in frameworks -%}
                    "{{ it }}"{% if not loop.last %}, {% endif %}
                    {%- endfor -%}])
                {% endif %}
    """))

    conanfile_headeronly = Template(textwrap.dedent("""
        from conans import ConanFile

        class HeaderOnly(ConanFile):
            name = "{{ref.name}}"
            version = "{{ref.version}}"

            {% if requires %}
            requires =
                {%- for req in requires -%}
                "{{ req }}"{% if not loop.last %}, {% endif %}
                {%- endfor -%}
            {% endif %}

            def package_id(self):
                self.info.header_only()

            def package_info(self):
                # It may declare system libraries
                {% for it in libs_system %}
                self.cpp_info.libs.append("{{ it }}")
                {% endfor %}

                {% if system_libs %}
                self.cpp_info.system_libs = [
                    {%- for it in system_libs -%}
                    "{{ it }}"{% if not loop.last %}, {% endif %}
                    {%- endfor -%}]
                {% endif %}

                {% if frameworks %}
                self.cpp_info.frameworks.extend([
                    {%- for it in frameworks -%}
                    "{{ it }}"{% if not loop.last %}, {% endif %}
                    {%- endfor -%}])
                {% endif %}
    """))

    main_cpp = textwrap.dedent("""
        int main() {return 0;}
    """)

    @classmethod
    def setUpClass(cls):
        libZ_ref = ConanFileReference.loads("libZ/version")
        libH2_ref = ConanFileReference.loads("header2/version")
        libH_ref = ConanFileReference.loads("header/version")
        libA_ref = ConanFileReference.loads("libA/version")
        libB_ref = ConanFileReference.loads("libB/version")
        libC_ref = ConanFileReference.loads("libC/version")
        libD_ref = ConanFileReference.loads("libD/version")

        t = TestClient(path_with_spaces=False)
        cls._cache_folder = t.cache_folder
        t.save({
            'libZ/conanfile.py': cls.conanfile.render(
                ref=libZ_ref,
                libs_extra=["Z2"],
                libs_system=["system_assumed"],
                system_libs=["system_lib"],
                frameworks=["Carbon"]),
            'libH2/conanfile.py': cls.conanfile_headeronly.render(
                ref=libH2_ref,
                libs_system=["header2_system_assumed"],
                system_libs=["header2_system_lib"],
                frameworks=["Security"]),
            'libH/conanfile.py': cls.conanfile_headeronly.render(
                ref=libH_ref,
                requires=[libH2_ref, libZ_ref],
                libs_system=["header_system_assumed"],
                system_libs=["header_system_lib"],
                frameworks=["CoreAudio"]),
            'libA/conanfile.py': cls.conanfile.render(
                ref=libA_ref,
                requires=[libH_ref],
                libs_extra=["A2"],
                libs_system=["system_assumed"],
                system_libs=["system_lib"],
                frameworks=["Carbon"]),
            'libB/conanfile.py': cls.conanfile.render(
                ref=libB_ref,
                requires=[libA_ref],
                libs_extra=["B2"],
                libs_system=["system_assumed"],
                system_libs=["system_lib"],
                frameworks=["Carbon"]),
            'libC/conanfile.py': cls.conanfile.render(
                ref=libC_ref,
                requires=[libA_ref],
                libs_extra=["C2"],
                libs_system=["system_assumed"],
                system_libs=["system_lib"],
                frameworks=["Carbon"]),
            'libD/conanfile.py': cls.conanfile.render(
                ref=libD_ref,
                requires=[libB_ref, libC_ref],
                libs_extra=["D2"],
                libs_system=["system_assumed"],
                system_libs=["system_lib"],
                frameworks=["Carbon"]),
        })

        # Create all of them
        t.run("create libZ")
        t.run("create libH2")
        t.run("create libH")
        t.run("create libA")
        t.run("create libB")
        t.run("create libC")
        t.run("create libD")

    def _validate_link_order(self, libs):
        # Check that all the libraries are there:
        self.assertEqual(len(libs), 19 if platform.system() == "Darwin" else
                         16 if platform.system() == "Linux" else 26,
                         msg="Unexpected number of libs ({}):"
                             " '{}'".format(len(libs), "', '".join(libs)))
        # - Regular libs
        ext = ".lib" if platform.system() == "Windows" else ".a"
        prefix = "" if platform.system() == "Windows" else "lib"
        expected_libs = {prefix + it + ext for it in ['libD', 'D2', 'libB', 'B2', 'libC', 'C2',
                                                      'libA', 'A2', 'libZ', 'Z2']}
        # - System libs
        ext_system = ".lib" if platform.system() == "Windows" else ""
        expected_libs.update([it + ext_system for it in ['header_system_assumed',
                                                         'header_system_lib',
                                                         'header2_system_assumed',
                                                         'header2_system_lib',
                                                         'system_assumed',
                                                         'system_lib']])
        # - Add MacOS frameworks
        if platform.system() == "Darwin":
            expected_libs.update(['CoreAudio', 'Security', 'Carbon'])
        # - Add Windows libs
        if platform.system() == "Windows":
            expected_libs.update(['kernel32.lib', 'user32.lib', 'gdi32.lib', 'winspool.lib',
                                  'shell32.lib', 'ole32.lib', 'oleaut32.lib', 'uuid.lib',
                                  'comdlg32.lib', 'advapi32.lib'])
        self.assertSetEqual(set(libs), expected_libs)

        # These are the first libraries and order is mandatory
        mandatory_1 = [prefix + it + ext for it in ['libD', 'D2', 'libB', 'B2', 'libC',
                                                    'C2', 'libA', 'A2', ]]
        self.assertListEqual(mandatory_1, libs[:len(mandatory_1)])

        # Then, libZ ones must be before system libraries that are consuming
        self.assertLess(libs.index(prefix + 'libZ' + ext),
                        min(libs.index('system_assumed' + ext_system),
                            libs.index('system_lib' + ext_system)))
        self.assertLess(libs.index(prefix + 'Z2' + ext),
                        min(libs.index('system_assumed' + ext_system),
                            libs.index('system_lib' + ext_system)))
        if platform.system() == "Darwin":
            self.assertLess(libs.index('liblibZ.a'), libs.index('Carbon'))
            self.assertLess(libs.index('libZ2.a'), libs.index('Carbon'))

    @staticmethod
    def _get_link_order_from_cmake(content):
        libs = []
        for it in content.splitlines():
            # This is for Linux and Mac
            # Remove double spaces from output that appear in some platforms
            line = ' '.join(it.split())
            if 'main.cpp.o -o example' in line:
                _, links = line.split("main.cpp.o -o example")
                for it_lib in links.split():
                    if it_lib.startswith("-l"):
                        libs.append(it_lib[2:])
                    elif it_lib == "-framework":
                        continue
                    else:
                        try:
                            _, libname = it_lib.rsplit('/', 1)
                        except ValueError:
                            libname = it_lib
                        finally:
                            libs.append(libname)
                break
            # Windows
            if 'example.exe" /INCREMENTAL:NO /NOLOGO' in it:
                for it_lib in it.split():
                    it_lib = it_lib.strip()
                    if it_lib.endswith(".lib"):
                        try:
                            _, libname = it_lib.rsplit('\\', 1)
                        except ValueError:
                            libname = it_lib
                        finally:
                            libs.append(libname)
                break
        return libs

    @staticmethod
    def _get_link_order_from_xcode(content):
        libs = []
        start_key = 'OTHER_LDFLAGS = (" -Wl,-search_paths_first -Wl,-headerpad_max_install_names",'
        end_key = '");'
        libs_content = content.split(start_key, 1)[1].split(end_key, 1)[0]
        libs_unstripped = libs_content.split(",")
        for lib in libs_unstripped:
            if ".a" in lib:
                libs.append(lib.strip('"').rsplit('/', 1)[1])
            elif "-l" in lib:
                libs.append(lib.strip('"')[2:])
            elif "-framework" in lib:
                libs.append(lib.strip('"')[11:])
        return libs

    def _create_find_package_project(self, multi):
        generator = "cmake_find_package_multi" if multi else "cmake_find_package"
        t = TestClient(cache_folder=self._cache_folder)
        t.save({
            'conanfile.txt': textwrap.dedent("""
                [requires]
                libD/version
                [generators]
                {}
                """.format(generator)),
            'CMakeLists.txt': textwrap.dedent("""
                cmake_minimum_required(VERSION 2.8.12)
                project(executable CXX)

                find_package(libD)
                add_executable(example main.cpp)
                target_link_libraries(example libD::libD)
                """),
            'main.cpp': self.main_cpp
        })

        t.run("install . -s build_type=Release")
        return t

    def _create_cmake_project(self, multi):
        generator = "cmake_multi" if multi else "cmake"
        include_cmake_file = "conanbuildinfo_multi" if multi else "conanbuildinfo"
        t = TestClient(cache_folder=self._cache_folder)
        t.save({
            'conanfile.txt': textwrap.dedent("""
                [requires]
                libD/version
                [generators]
                {}
                """.format(generator)),
            'CMakeLists.txt': textwrap.dedent("""
                cmake_minimum_required(VERSION 2.8.12)
                project(executable CXX)

                include(${{CMAKE_BINARY_DIR}}/{}.cmake)
                conan_basic_setup(TARGETS NO_OUTPUT_DIRS)

                add_executable(example main.cpp)
                target_link_libraries(example CONAN_PKG::libD)
                """.format(include_cmake_file)),
            'main.cpp': self.main_cpp
        })

        t.run("install . -s build_type=Release")
        t.save({"conanbuildinfo_debug.cmake": "# just be there"})
        return t

    def _run_and_get_lib_order(self, t, generator, find_package_config=False):
        extra_cmake = "-DCMAKE_PREFIX_PATH=." if find_package_config else "-DCMAKE_MODULE_PATH=."
        if generator == "Xcode":
            t.run_command("cmake . -G Xcode {} -DCMAKE_VERBOSE_MAKEFILE:BOOL=True"
                          " -DCMAKE_CONFIGURATION_TYPES=Release".format(extra_cmake))
            t.run_command("cmake --build .", assert_error=True)
            # Get the actual link order from the CMake call
            libs = self._get_link_order_from_xcode(t.load(os.path.join('executable.xcodeproj',
                                                                       'project.pbxproj')))
        else:
            t.run_command("cmake . {} -DCMAKE_VERBOSE_MAKEFILE:BOOL=True"
                          " -DCMAKE_BUILD_TYPE=Release".format(extra_cmake))
            extra_build = "--config Release" if platform.system() == "Windows" else ""  # Windows VS
            t.run_command("cmake --build . {}".format(extra_build), assert_error=True)
            # Get the actual link order from the CMake call
            libs = self._get_link_order_from_cmake(str(t.out))
        return libs

    @parameterized.expand([(None,), ("Xcode",)])
    def test_cmake_find_package_multi(self, generator):
        if generator == "Xcode" and platform.system() != "Darwin":
            self.skipTest("Xcode is needed")

        t = self._create_find_package_project(multi=True)
        libs = self._run_and_get_lib_order(t, generator, find_package_config=True)
        self._validate_link_order(libs)

    @parameterized.expand([(None,), ("Xcode",)])
    def test_cmake_find_package(self, generator):
        if generator == "Xcode" and platform.system() != "Darwin":
            self.skipTest("Xcode is needed")

        t = self._create_find_package_project(multi=False)
        libs = self._run_and_get_lib_order(t, generator)
        self._validate_link_order(libs)

    @parameterized.expand([(None,), ("Xcode",)])
    def test_cmake(self, generator):
        if generator == "Xcode" and platform.system() != "Darwin":
            self.skipTest("Xcode is needed")

        t = self._create_cmake_project(multi=False)
        libs = self._run_and_get_lib_order(t, generator)
        self._validate_link_order(libs)

    @parameterized.expand([(None,), ("Xcode",)])
    def test_cmake_multi(self, generator):
        if generator == "Xcode" and platform.system() != "Darwin":
            self.skipTest("Xcode is needed")

        t = self._create_cmake_project(multi=True)
        libs = self._run_and_get_lib_order(t, generator)
        self._validate_link_order(libs)



import pytest

from conans.client.build.cmake_flags import get_generator
from conans.test.utils.mocks import ConanFileMock, MockSettings


@pytest.mark.parametrize("compiler,version,expected", [
    ("Visual Studio", "15", "Visual Studio 15 2017"),
    ("Visual Studio", "15.9", "Visual Studio 15 2017"),
    ("msvc", "193", "Visual Studio 17 2022"),
    ("msvc", "192", "Visual Studio 16 2019")
])
def test_vs_generator(compiler, version, expected):
    settings = MockSettings({"os": "Windows", "arch": "x86_64", "compiler": compiler})
    conanfile = ConanFileMock()
    conanfile.settings = settings

    settings.values['compiler.version'] = version
    assert get_generator(conanfile) == expected



import os
import platform
import textwrap

import pytest
import six

from conans.model.ref import ConanFileReference
from conans.test.assets.genconanfile import GenConanfile
from conans.test.utils.tools import TestClient, TurboTestClient
from conans.util.files import save, load


@pytest.fixture
def conanfile():
    conanfile = str(GenConanfile().with_import("from conans import tools")
                    .with_import("import os")
                    .with_setting("build_type").with_setting("arch")
                    .with_import("from conan.tools.layout import {ly}")
                    .with_import("from conan.tools.files import AutoPackager"))

    conanfile += """
    def source(self):
        tools.save("include/myheader.h", "")

    def build(self):
        tools.save("{libpath}/mylib.lib", "")

    def layout(self):
        {ly}(self)

    def package(self):
        AutoPackager(self).run()
    """
    return conanfile


subfolders_arch = {"armv7": "ARM", "armv8": "ARM64", "x86": None, "x86_64": "x64"}


@pytest.mark.parametrize("arch", ["x86_64", "x86", "armv7", "armv8"])
@pytest.mark.parametrize("build_type", ["Debug", "Release"])
def test_layout_in_cache(conanfile, build_type, arch):
    """The layout in the cache is used too, always relative to the "base" folders that the cache
    requires. But by the default, the "package" is not followed
    """
    client = TurboTestClient()

    ref = ConanFileReference.loads("lib/1.0")
    libarch = subfolders_arch.get(arch)
    libpath = "{}{}".format(libarch + "/" if libarch else "", build_type)
    pref = client.create(ref, args="-s arch={} -s build_type={}".format(arch, build_type),
                         conanfile=conanfile.format(ly="vs_layout", libpath=libpath))
    bf = client.cache.package_layout(pref.ref).build(pref)
    pf = client.cache.package_layout(pref.ref).package(pref)

    # Check the build folder
    assert os.path.exists(os.path.join(os.path.join(bf, libpath), "mylib.lib"))

    # Check the package folder
    assert os.path.exists(os.path.join(pf, "lib/mylib.lib"))
    assert os.path.exists(os.path.join(pf, "include", "myheader.h"))


@pytest.mark.parametrize("arch", ["x86_64", "x86", "armv7", "armv8"])
@pytest.mark.parametrize("build_type", ["Debug", "Release"])
def test_layout_with_local_methods(conanfile, build_type, arch):
    """The layout in the cache is used too, always relative to the "base" folders that the cache
        requires. But by the default, the "package" is not followed
        """
    client = TestClient()
    libarch = subfolders_arch.get(arch)
    libpath = "{}{}".format(libarch + "/" if libarch else "", build_type)
    client.save({"conanfile.py": conanfile.format(ly="vs_layout", libpath=libpath)})
    client.run("install . lib/1.0@ -s build_type={} -s arch={}".format(build_type, arch))
    client.run("source .")
    # Check the source folder (release)
    assert os.path.exists(os.path.join(client.current_folder, "include", "myheader.h"))
    client.run("build .")
    # Check the build folder (release)
    assert os.path.exists(os.path.join(os.path.join(client.current_folder, libpath), "mylib.lib"))

    # Check the package
    client.run("package .", assert_error=True)
    assert "The usage of the 'conan package' local method is disabled when using layout()" \
           "" in client.out


@pytest.mark.skipif(platform.system() != "Windows", reason="Removing msvc compiler")
def test_error_no_msvc():
    # https://github.com/conan-io/conan/issues/9953
    conanfile = textwrap.dedent("""
        from conans import ConanFile
        from conan.tools.cmake import cmake_layout
        class Pkg(ConanFile):
            settings = "os", "compiler", "build_type", "arch"
            def layout(self):
                cmake_layout(self)
        """)
    settings_yml = textwrap.dedent("""
        os: [Windows]
        os_build: [Windows]
        arch_build: [x86_64]
        compiler:
            gcc:
                version: ["8"]
        build_type: [Release]
        arch: [x86_64]
        """)
    client = TestClient()
    client.save({"conanfile.py": conanfile})
    save(client.cache.settings_path, settings_yml)
    client.run('install . -s os=Windows -s build_type=Release -s arch=x86_64 '
               '-s compiler=gcc -s compiler.version=8')
    assert "Installing" in client.out


def test_error_no_build_type():
    # https://github.com/conan-io/conan/issues/9953
    conanfile = textwrap.dedent("""
        from conans import ConanFile
        from conan.tools.cmake import cmake_layout
        class Pkg(ConanFile):
            settings = "os", "compiler", "arch"
            def layout(self):
                cmake_layout(self)
        """)

    client = TestClient()
    client.save({"conanfile.py": conanfile})
    client.run('install .',  assert_error=True)
    assert " 'build_type' setting not defined, it is necessary for cmake_layout()" in client.out


@pytest.mark.skipif(six.PY2, reason="only Py3")
def test_cmake_layout_external_sources():
    conanfile = textwrap.dedent("""
        import os
        from conans import ConanFile
        from conan.tools.cmake import cmake_layout
        from conan.tools.files import save, copy, load
        class Pkg(ConanFile):
            settings = "os", "build_type"
            exports_sources = "exported.txt"

            def layout(self):
                cmake_layout(self, src_folder="src")

            def generate(self):
                save(self, "generate.txt", "generate")

            def source(self):
                save(self, "source.txt", "foo")

            def build(self):
                c1 = load(self, os.path.join(self.source_folder, "source.txt"))
                c2 = load(self, os.path.join(self.source_folder, "..", "exported.txt"))
                save(self, "build.txt", c1 + c2)

            def package(self):
                copy(self, "build.txt", self.build_folder, os.path.join(self.package_folder, "res"))
        """)

    client = TestClient()
    client.save({"conanfile.py": conanfile, "exported.txt": "exported_contents"})
    client.run("create . foo/1.0@ -s os=Linux")
    assert "Packaged 1 '.txt' file: build.txt" in client.out

    # Local flow
    client.run("install . foo/1.0 -s os=Linux")
    assert os.path.exists(os.path.join(client.current_folder, "build", "generators", "generate.txt"))
    client.run("source .")
    assert os.path.exists(os.path.join(client.current_folder, "src", "source.txt"))
    client.run("build .")
    contents = load(os.path.join(client.current_folder, "build", "Release", "build.txt"))
    assert contents == "fooexported_contents"
    client.run("export-pkg . foo/1.0@ --force")
    assert "Packaged 1 '.txt' file: build.txt" in client.out


@pytest.mark.skipif(six.PY2, reason="only Py3")
@pytest.mark.parametrize("with_build_type", [True, False])
def test_basic_layout_external_sources(with_build_type):
    conanfile = textwrap.dedent("""
            import os
            from conans import ConanFile
            from conan.tools.layout import basic_layout
            from conan.tools.files import save, load, copy
            class Pkg(ConanFile):
                settings = "os", "compiler", "arch"{}
                exports_sources = "exported.txt"

                def layout(self):
                    basic_layout(self, src_folder="src")

                def generate(self):
                    save(self, "generate.txt", "generate")

                def source(self):
                    save(self, "source.txt", "foo")

                def build(self):
                    c1 = load(self, os.path.join(self.source_folder, "source.txt"))
                    c2 = load(self, os.path.join(self.source_folder, "..", "exported.txt"))
                    save(self, "build.txt", c1 + c2)

                def package(self):
                    copy(self, "build.txt", self.build_folder, os.path.join(self.package_folder, "res"))
            """)
    if with_build_type:
        conanfile = conanfile.format(', "build_type"')
    else:
        conanfile = conanfile.format("")
    client = TestClient()
    client.save({"conanfile.py": conanfile, "exported.txt": "exported_contents"})
    client.run("create . foo/1.0@ -s os=Linux")
    assert "Packaged 1 '.txt' file: build.txt" in client.out

    # Local flow
    build_folder = "build-release" if with_build_type else "build"
    client.run("install . foo/1.0 -s os=Linux")
    assert os.path.exists(os.path.join(client.current_folder, build_folder, "conan", "generate.txt"))
    client.run("source .")
    assert os.path.exists(os.path.join(client.current_folder, "src", "source.txt"))
    client.run("build .")
    contents = load(os.path.join(client.current_folder, build_folder, "build.txt"))
    assert contents == "fooexported_contents"
    client.run("export-pkg . foo/1.0@ --force")
    assert "Packaged 1 '.txt' file: build.txt" in client.out


@pytest.mark.skipif(six.PY2, reason="only Py3")
@pytest.mark.parametrize("with_build_type", [True, False])
def test_basic_layout_no_external_sources(with_build_type):
    conanfile = textwrap.dedent("""
            import os
            from conans import ConanFile
            from conan.tools.layout import basic_layout
            from conan.tools.files import save, load, copy
            class Pkg(ConanFile):
                settings = "os", "compiler", "arch"{}
                exports_sources = "exported.txt"

                def layout(self):
                    basic_layout(self)

                def generate(self):
                    save(self, "generate.txt", "generate")

                def build(self):
                    contents = load(self, os.path.join(self.source_folder, "exported.txt"))
                    save(self, "build.txt", contents)

                def package(self):
                    copy(self, "build.txt", self.build_folder, os.path.join(self.package_folder,
                                                                            "res"))
            """)
    if with_build_type:
        conanfile = conanfile.format(', "build_type"')
    else:
        conanfile = conanfile.format("")

    client = TestClient()
    client.save({"conanfile.py": conanfile, "exported.txt": "exported_contents"})
    client.run("create . foo/1.0@ -s os=Linux")
    assert "Packaged 1 '.txt' file: build.txt" in client.out

    # Local flow
    client.run("install . foo/1.0 -s os=Linux")

    build_folder = "build-release" if with_build_type else "build"
    assert os.path.exists(os.path.join(client.current_folder, build_folder, "conan", "generate.txt"))

    client.run("build .")
    contents = load(os.path.join(client.current_folder, build_folder, "build.txt"))
    assert contents == "exported_contents"
    client.run("export-pkg . foo/1.0@ --force")
    assert "Packaged 1 '.txt' file: build.txt" in client.out


def test_cmake_layout_custom_build_folder():
    # https://github.com/conan-io/conan/issues/11838
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.cmake import cmake_layout
        class Pkg(ConanFile):
            settings = "os", "build_type"
            generators = "CMakeToolchain"
            def layout(self):
                cmake_layout(self, src_folder="src", build_folder="mybuild")
        """)

    client = TestClient()
    client.save({"conanfile.py": conanfile})
    client.run("install .")
    assert os.path.exists(os.path.join(client.current_folder,
                                       "mybuild/generators/conan_toolchain.cmake"))
